package at.bestsolution.drawswf;

import java.util.Vector;
import java.util.LinkedList;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.net.URL;
import java.util.ResourceBundle;
import javax.swing.*;
import java.awt.Insets;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.*;

import at.bestsolution.drawswf.drawobjects.*;
import at.bestsolution.drawswf.MainWindow;

/**
 * @author tom and Mike Lam and Taras Kowaliw
 */

public class DrawObjectList extends JPanel implements ListSelectionListener, ActionListener, ItemListener
{
    public DefaultListModel list_model_;
    private JPanel whiteBoardDesc;
    private PropertiesDialog properties_;
    public JComboBox actionList = new JComboBox();

    public Vector SVGDescriptions = new Vector();
    public Vector descriptions = new Vector();
    public Vector objectList = new Vector();
    public Vector lockedDescriptions = new Vector();

    // Stuff at the bottom (descriptionEditingPanel)
    private JButton submitDesc;
    private JButton lock;
    private JButton unlock;
    private JTextArea objectDescriptionArea;
    private JList draw_object_list_;

    private MainWindow mWin;

    private String myLockedObjectStr = "";
    private String mySelectedObjectStr = "";

    protected static final String tool_bar_icon_path_ = "at/bestsolution/drawswf/images/16x16/";


    // ******************************************************************
    // ******************************************************************
    // ******************************************************************
    // ****************** CONSTRUCTOR & HELPERS *************************
    public DrawObjectList(MainWindow mW)
    {
        super();

        mWin = mW;
        whiteBoardDesc = this;
        whiteBoardDesc.setPreferredSize(new Dimension(350, 35));
        properties_ = null;

        list_model_ = new DefaultListModel();
        draw_object_list_ = new JList(list_model_);
        draw_object_list_.addListSelectionListener(this);
        draw_object_list_.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        addComponents();
    }

    public int getNum() {
        int num = mWin.getDrawingPanel().getNewNumber();
        if (mWin.isWaiting()) return num + 1;
        return num;
    }

    public JPanel addEditDescriptionsPanel() {
        JPanel descriptionEditingPanel = new JPanel();
        descriptionEditingPanel.setLayout(new BorderLayout());
        descriptionEditingPanel.setPreferredSize(new Dimension(350, 150));
        descriptionEditingPanel.setBorder(BorderFactory.createTitledBorder("Descriptions for Whiteboard Objects"));

        JScrollPane objectsWithoutDescription = new JScrollPane();
        objectsWithoutDescription.setPreferredSize(new Dimension(320, 50));
        objectsWithoutDescription.setViewportView(draw_object_list_);

        ImageIcon icon;
        URL icon_url;
        icon_url = getClass().getClassLoader().getResource(tool_bar_icon_path_+ "lock.png");
        icon = new ImageIcon(icon_url);
        lock = new JButton(icon);
        lock.setActionCommand("lock");
        lock.setToolTipText("Lock Description");
        lock.setMargin(new Insets(1,1,1,1));
        lock.addActionListener(this);

        icon_url = getClass().getClassLoader().getResource(tool_bar_icon_path_+ "unlock.png");
        icon = new ImageIcon(icon_url);
        unlock = new JButton(icon);
        unlock.setActionCommand("unlock");
        unlock.setToolTipText("Unlock Description");
        unlock.setMargin(new Insets(1,1,1,1));
        unlock.addActionListener(this);

        icon_url = getClass().getClassLoader().getResource(tool_bar_icon_path_+ "delete.png");
        icon = new ImageIcon(icon_url);
        JButton delete = new JButton(icon);
        delete.setActionCommand("deleteDesc");
        delete.setToolTipText("No Description");
        delete.setMargin(new Insets(1,1,1,1));
        delete.addActionListener(this);

        icon_url = getClass().getClassLoader().getResource(tool_bar_icon_path_+ "send.gif");
        icon = new ImageIcon(icon_url);
        submitDesc = new JButton(icon);
        submitDesc.setActionCommand("submitDesc");
        submitDesc.setToolTipText("Submit Description");
        submitDesc.setMargin(new Insets(1,1,1,1));
        submitDesc.addActionListener(this);

        objectDescriptionArea = new JTextArea();
        objectDescriptionArea.setFocusable(true);
        objectDescriptionArea.setLineWrap(true);

        JScrollPane descriptionArea = new JScrollPane(objectDescriptionArea);
        descriptionArea.setPreferredSize(new Dimension(320, 50));
        objectDescriptionArea.addKeyListener(new KeyListener() {
            public void keyTyped(KeyEvent e) { }
            public void keyReleased(KeyEvent e) {
                DescriptionHandler dh = getCurrentDH();
                String description = objectDescriptionArea.getText().trim();
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    if(dh != null)
                        sendDescription(dh,description);
                }
            }
            public void keyPressed(KeyEvent e) { }
        });

        JPanel topPanel = new JPanel();
        topPanel.setLayout(new BorderLayout());
        topPanel.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
        JPanel bottomPanel = new JPanel();
        bottomPanel.setLayout(new BorderLayout());
        bottomPanel.setBorder(BorderFactory.createEmptyBorder(0,1,0,1));

        JPanel buttonPanel2 = new JPanel();
        buttonPanel2.setLayout(new BoxLayout(buttonPanel2, BoxLayout.Y_AXIS));
        buttonPanel2.add(Box.createVerticalGlue());
        buttonPanel2.add(lock);
        buttonPanel2.add(unlock);
        buttonPanel2.add(delete);

        JPanel lowerButtonPanel = new JPanel();
        lowerButtonPanel.setLayout(new BoxLayout(lowerButtonPanel, BoxLayout.Y_AXIS));
        lowerButtonPanel.add(Box.createVerticalGlue());
        lowerButtonPanel.add(submitDesc);

        topPanel.add(buttonPanel2, "East");
        topPanel.add(objectsWithoutDescription, "West");

        bottomPanel.add(descriptionArea, "West");
        bottomPanel.add(lowerButtonPanel, "East");

        descriptionEditingPanel.add(topPanel, "North");
        descriptionEditingPanel.add(bottomPanel, "Center");

        return descriptionEditingPanel;
    }


    // **************************************************************
    // **************************************************************
    // **************** ADDING & UPDATING SVG ***********************
    private void addComponents() {
        JPanel descriptionPanel = new JPanel();
        actionList.setPreferredSize(new Dimension(345, 20));
        actionList.addItemListener(this);
        descriptionPanel.add(actionList);
        whiteBoardDesc.add(descriptionPanel, BorderLayout.CENTER);
    }

    public void addDescriptionFromPalette(DescriptionHandler dh, DrawObject drawObject)
    {
        DescriptionHandler desc = (DescriptionHandler)descriptions.elementAt(descriptions.size()-1);
        descriptions.remove(desc);
        descriptions.add(dh);

        if(dh.getObjectStatus().equals("open"))
        {
            list_model_.addElement(drawObject.getName());
            sendNewDescriptionFromPalette(dh, dh.getObjectDescription());
        }
        else
            sendDescription(dh,dh.getObjectDescription());

        refreshDescriptionBox();
    }

    public void loadDescriptions(Vector descObjects)
    {
        boolean hasMatch = false;
        int currDescSize = descriptions.size();
        actionList.removeAllItems();

        for(int i = 0; i < descObjects.size(); i++)
        {
            String description = (String)descObjects.elementAt(i);
            DescriptionHandler dh = new DescriptionHandler(description);

            if(dh.getObjectDescription().equals(""))
                actionList.addItem(dh.getObjectName());
            else
                actionList.addItem(dh.getObjectName() + " - " + dh.getObjectDescription());

            if(dh.getObjectStatus().equals("lock")) {
                lockedDescriptions.add(dh);
                list_model_.removeElement(dh.getObjectName());
            }

            if(dh.getObjectStatus().equals("set"))
                list_model_.removeElement(dh.getObjectName());

            for(int j = 0; j < currDescSize; j++) {
                DescriptionHandler existingDH = (DescriptionHandler)descriptions.elementAt(j);

                if(existingDH.getObjectName().equals(dh.getObjectName())) {
                    existingDH.setObjectDescription(dh.getObjectDescription());
                    existingDH.setObjectStatus(dh.getObjectStatus());
                    hasMatch = true;
                }
            }

            if(hasMatch == false)
                descriptions.add(dh);

            if(list_model_.size() > 0)
                draw_object_list_.setSelectedIndex(0);

            doSelection();
        }
    }

    public void addDrawObject(DrawObject draw_object)
    {
        addDrawObject(draw_object, false);
    }

    public void addDrawObject(DrawObject draw_object, boolean isNew)
    {
        list_model_.addElement(draw_object.getName());
        draw_object_list_.setSelectedValue(draw_object.getName(), true);
        objectList.add(draw_object);

        DescriptionHandler dh = new DescriptionHandler(draw_object.getName(), "", "open");
        descriptions.add(dh);

        if (isNew) {
            actionList.addItem(draw_object.getName());

            String theName = (String) draw_object_list_.getSelectedValue();
            String message = "";
            String name = draw_object.getName().substring(0, draw_object.getName().indexOf(" "));
            String text = "";

            setActionListSelected(theName);

            if(name.equals("Text")) {
                Text textObj = (Text)draw_object;
                text = (String)textObj.getPropertyText();
                dh.setObjectDescription(text);
                dh.setObjectStatus("set");
                list_model_.removeElement(dh.getObjectName());
                refreshDescriptionBox();
                message = mWin.myUserName + " draws " + name + ": "+text;
            }

            System.out.println("addDrawObject - isNew!");
            mWin.sendModWhiteboard(message, draw_object);
        }

        mySelectedObjectStr = getActionListSelected();
    }


    // **************************************************************
    // **************************************************************
    // **************** EXPORTING SVG ***********************
    public Vector getAllDescriptionsSVG() {
        SVGDescriptions.clear();

        for(int i = 0; i < descriptions.size(); i++) {
            DescriptionHandler dh = (DescriptionHandler)descriptions.elementAt(i);
            String SVGDescription = dh.getXMLString();
            SVGDescriptions.add(SVGDescription);
        }

        return SVGDescriptions;
    }


    // **************************************************************************
    // **************************************************************************
    // **************************************************************************
    // ********************* UPDATES TO SELECTION AREAS ***********************

    // ActionList (the combo box)  has been changed
    public void itemStateChanged(ItemEvent e) {
        updateDescriptionBox();
        showSelectedObject();
        mWin.getDrawingPanel().repaint();
        handleDescArea();
        mySelectedObjectStr = getActionListSelected();
    }

    // draw_object_list_ (the list) has been changed
    public void valueChanged(ListSelectionEvent e) {
        String theName = (String) draw_object_list_.getSelectedValue();
        actionList.setSelectedItem(theName);

        if ((mWin.getDrawingPanel().getDrawingList().size() > 0) && (draw_object_list_.getSelectedIndex() != -1)) {
            if (properties_ == null) {
                properties_ = new PropertiesDialog(mWin.getDrawingPanel().getDrawingList().get(draw_object_list_.getSelectedIndex()));
            } else {
                properties_.setDrawObject(mWin.getDrawingPanel().getDrawingList().get(draw_object_list_.getSelectedIndex()));
            }
        }

        mySelectedObjectStr = getActionListSelected();
    }


    // **************************************************************************
    // **************************************************************************
    // **************************************************************************
    // ********************* GETTING & PUTTING SELECTIONS ***********************
    public void setSelectedObject(DrawObject new_focus) {
        String tempName = (String) draw_object_list_.getSelectedValue();
        if (!(tempName != null && tempName.equals(myLockedObjectStr))) {
            draw_object_list_.setSelectedValue(new_focus.getName(), true);
        }
        setActionListSelected(new_focus.getName());
        mySelectedObjectStr = getActionListSelected();
        showSelectedObject();
    }

    public String getSelectedObject(){
        return getActionListSelected();
    }

    // selects either the object specified by mySelectedObjectStr, or the 0th item
    public void doSelection() {
        if (mySelectedObjectStr == null || mySelectedObjectStr.length() < 1) {
            mySelectedObjectStr = getActionListItemName(0);
        }
        if (mySelectedObjectStr != null) {
            setActionListSelected(mySelectedObjectStr);
            updateDescriptionBox();
            showSelectedObject();
        }
    }

    public DescriptionHandler getCurrentDH() {
        DescriptionHandler dH;

        if (draw_object_list_.getSelectedValue() == null)
            return null;

        for(int i=0; i<descriptions.size(); i++) {
            dH = (DescriptionHandler)descriptions.elementAt(i);
            if(((String)draw_object_list_.getSelectedValue()).equals(dH.getObjectName())) return dH;
        }
        return null;
    }


    // ******************************************************************
    // ******************************************************************
    // ************* FUNCTIONS TO ACCESS ACTIONLIST *********************
    public String removeDescription(String raw) {
        int descStart = raw.indexOf(" - ");
        if (descStart < 0)
            return raw;
        return raw.substring(0,descStart);
    }

    public String getActionListItemName(int index) {
        String rawName = (String) actionList.getItemAt(index);
        if (rawName == null)
            return null;

        return removeDescription(rawName);
    }

    public boolean isInActionList(String name) {
        for (int ct=0; ct< actionList.getItemCount(); ct++) {
            String currName = (String) actionList.getItemAt(ct);
            if (removeDescription(currName).equals(name)) {
                return true;
            }
        }
        return false;
    }

    public String getActionListSelected() {
        String rawName = (String) actionList.getSelectedItem();
        if (rawName == null) return null;
            return removeDescription(rawName);
    }

    public void setActionListSelected(String name) {
        for (int ct=0; ct< actionList.getItemCount(); ct++) {
            String currName = (String) actionList.getItemAt(ct);
            if (removeDescription(currName).equals(name)) {
                actionList.setSelectedIndex(ct);
            }
        }
    }

    public void deleteActionListItem(String name) {
        for (int ct=0; ct< actionList.getItemCount(); ct++) {
            String currName = (String) actionList.getItemAt(ct);
            if (removeDescription(currName).equals(name)) {
                actionList.removeItemAt(ct);
            }
        }
    }


    // ******************************************************************
    // ******************************************************************
    // ******************************************************************
    // ******************************************************************
    // ********* FUNCTIONS TO UPDATE STUFF AFTER SELECTION **************
    public void showSelectedObject() {
        double xPt = 0;
        double yPt = 0;
        DrawObject obj = null;
        String name = getActionListSelected();

        if (name != null) {
            for (int i = 0; i < objectList.size(); i++) {
                obj = (DrawObject)objectList.elementAt(i);

                if(obj != null) {
                    if (name.equals(obj.getName())) {
                        xPt = obj.getPropertyX();
                        yPt = obj.getPropertyY();
                        break;
                    }
                }
            }

            mWin.getDrawingPanel().updateObjectPointer(xPt,yPt);
        }
    }

    public void refreshDescriptionBox() {
        DescriptionHandler dh;
        actionList.removeAllItems();

        for(int i = 0; i < descriptions.size(); i++) {
            dh = (DescriptionHandler)descriptions.elementAt(i);

            if(dh.getObjectDescription().equals(""))
                actionList.addItem(dh.getObjectName());
            else
                actionList.addItem(dh.getObjectName() + " - " + dh.getObjectDescription());
        }
    }

    public void updateDescriptionBox() {
        String currentObject = getActionListSelected();
        if (currentObject == null) return;
        String tempName = (String) draw_object_list_.getSelectedValue();
        if (tempName != null && tempName.equals(myLockedObjectStr)) return;

        for (int i = 0; i < list_model_.size(); i++) {
            if(currentObject.equals((String)list_model_.getElementAt(i))) {
                draw_object_list_.setSelectedIndex(i);
            }
        }
    }

    // A function which either greys out or makes active
    // the description editing box and sendDesc button
    public void handleDescArea() {
        DescriptionHandler dH = getCurrentDH();

        if (dH != null && dH.getObjectStatus().equalsIgnoreCase("lock") && myLockedObjectStr.equalsIgnoreCase(dH.getObjectName())) {
            objectDescriptionArea.setEditable(true);
            submitDesc.setEnabled(true);
            unlock.setEnabled(true);
            lock.setEnabled(false);
            draw_object_list_.setEnabled(false);
        } else if (dH != null && dH.getObjectStatus().equalsIgnoreCase("lock")) {
            objectDescriptionArea.setText("");
            objectDescriptionArea.setEditable(false);
            submitDesc.setEnabled(false);
            unlock.setEnabled(false);
            lock.setEnabled(false);
            draw_object_list_.setEnabled(true);
        } else {
            objectDescriptionArea.setText("");
            objectDescriptionArea.setEditable(false);
            submitDesc.setEnabled(false);
            unlock.setEnabled(false);
            lock.setEnabled(true);
            draw_object_list_.setEnabled(true);
       }
    }

    public void sendDescription(DescriptionHandler dh, String description) {
        dh.setObjectDescription(description);
        dh.setObjectStatus("set");
        list_model_.removeElement(dh.getObjectName());
        objectDescriptionArea.setText("");

        if(list_model_.size() >= 1)
            draw_object_list_.setSelectedIndex(list_model_.size()-1);

        String mStr = mWin.myUserName + " describes " + dh.getObjectName() + ": " + description;
        refreshDescriptionBox();
        mWin.sendModWhiteboardDesc(mStr, dh);
        myLockedObjectStr = "";
    }

    public void sendNewDescriptionFromPalette(DescriptionHandler dh, String description)
    {
        dh.setObjectDescription(description);
        dh.setObjectStatus("set");
        list_model_.removeElement(dh.getObjectName());
        objectDescriptionArea.setText("");

        if(list_model_.size() >= 1)
            draw_object_list_.setSelectedIndex(list_model_.size()-1);

        String mStr = mWin.myUserName + " adds " + dh.getObjectName() + ": No description";

        refreshDescriptionBox();
        mWin.sendModWhiteboardDesc(mStr, dh);
        myLockedObjectStr = "";
    }

    // ******************************************************************
    // ******************************************************************
    // ************* ACTIONS & HANDLERS & COMMANDS FROM MOD *************
    public void actionPerformed(ActionEvent e) {
        DescriptionHandler dh = getCurrentDH();
        DescriptionHandler lockedDesc = null;

        if(e.getActionCommand().equals("deleteDesc")) {
            deleteDescription();
        }

        if(e.getActionCommand().equals("clear")) {
            clearWhiteboard();
            // see clearWhiteboard() for mWin.sendWhiteboard
        }

        if(e.getActionCommand().equals("submitDesc")) {
            boolean isLockedDesc = false;
            String description = objectDescriptionArea.getText().trim();

            if(dh != null) {
                for(int i = 0; i < lockedDescriptions.size(); i++) {
                    lockedDesc = (DescriptionHandler)lockedDescriptions.elementAt(i);

                    if(dh.getObjectName().equals(lockedDesc.getObjectName())) {
                        //grey out box -- to do
                        isLockedDesc = true;
                        break;
                    }
                }

                if(isLockedDesc == false) {
                    if((description != null)&&(description.length() > 0)) {
                        if(dh.getObjectStatus().equals("lock")) {
                            sendDescription(dh,description);
                        }
                    }
                }

            }
            handleDescArea();
        }

        if(e.getActionCommand().equals("lock")) {
            if(dh != null) {
                dh.setObjectStatus("lock");
                myLockedObjectStr = dh.getObjectName();
                mWin.sendModWhiteboardDesc("", dh);
                handleDescArea();
            }
        }

        if(e.getActionCommand().equals("unlock")) {
            if(dh != null) {
                dh.setObjectStatus("open");
                objectDescriptionArea.setText("");
                myLockedObjectStr = "";
                mWin.sendModWhiteboardDesc("", dh);
            }
            handleDescArea();
        }
    }

    public void deleteDescription() {
        DescriptionHandler dh;
        String message = mWin.myUserName + " deletes " + getSelectedObject() + " description";

        if ((descriptions.size() != 0) && (getSelectedObject() != null)) {
           dh = getCurrentDH();

            if(dh != null) {
                list_model_.removeElement(getSelectedObject());
                dh.setObjectStatus("set");
                dh.setObjectDescription("");

                if (list_model_.size() >= 1) draw_object_list_.setSelectedIndex(list_model_.size()-1);

                refreshDescriptionBox();
                mWin.sendWhiteboard(message);
            }
        }
    }

    // called by the delete button in DrawFileToolbar
    public void deleteObjectAndSend() {
        String theObj = getActionListSelected();
        String mStr = "";
        if (theObj != null && theObj.length() > 0) {
            DrawObject tmp = mWin.getDrawingPanel().getObjectByName(theObj);

            if(tmp != null) {
                int selectedIndex = mWin.getDrawingPanel().getIndex(theObj);
                mWin.getDrawingPanel().moveDrawingObject(selectedIndex, 0);
                deleteActionListItem(theObj);
                objectList.remove(tmp);


                for(int j = 0; j < descriptions.size(); j++) {
                    DescriptionHandler existingDH = (DescriptionHandler)descriptions.elementAt(j);
                    if (existingDH.getObjectName().equals(theObj)) {
                if (existingDH.getObjectDescription() != null && existingDH.getObjectDescription().length() > 0)
                mStr = mWin.myUserName + " deletes " + theObj + ": " + existingDH.getObjectDescription();
                        descriptions.remove(j);
                        break;
                    }
                }
                mWin.sendModWhiteboardRM(mStr, theObj);
            }
        }
        doSelection();
    }

    // called by MainWindow for a MOD message
    public void modDeleteObject(String objName) {
        if (objName != null && objName.length() > 0) {
            DrawObject tmp = mWin.getDrawingPanel().getObjectByName(objName);

            if(tmp != null) {
                int selectedIndex = mWin.getDrawingPanel().getIndex(objName);
                mWin.getDrawingPanel().moveDrawingObject(selectedIndex, 0);
                deleteActionListItem(objName);
                objectList.remove(tmp);


                for(int j = 0; j < descriptions.size(); j++) {
                    DescriptionHandler existingDH = (DescriptionHandler)descriptions.elementAt(j);
                    if (existingDH.getObjectName().equals(objName)) {
                        descriptions.remove(j);
                        break;
                    }
                }
            }

            if (objName.equals(myLockedObjectStr)) {
                objectDescriptionArea.setText("Sorry, the item you were describing has been deleted.");
            }
        }
        doSelection();
    }

    public void clearWhiteboard() {
        int n = JOptionPane.showConfirmDialog(null, "<html><body><p>Are you sure you wish to clear the whiteboard?</p></body></html>", "AComm - Clear Whiteboard?", JOptionPane.YES_NO_OPTION);

        if (n == JOptionPane.YES_OPTION) {
            clearDrawObjects();
            String message = mWin.myUserName + " clears the whiteboard.";
            mWin.sendWhiteboard(message);
        }
    }

    public void removeDrawObject(DrawObject draw_object) {
        list_model_.removeElement(draw_object.getName());
    }

    public void moveDrawObject(int selected_index, int amount)
    {
        Object tmp = list_model_.get(selected_index);
        Object tmp2 = list_model_.set(selected_index + amount, tmp);
        list_model_.set(selected_index, tmp2);
        draw_object_list_.setSelectedIndex(selected_index + amount);
        doSelection();
    }

    public void clearDrawObjects() {
        try {
            actionList.removeAllItems();
            objectList.clear();
            list_model_.clear();
            descriptions.clear();
            mWin.getDrawingPanel().clearDrawing();
            resetObjectCounters();
            SVGDescriptions.clear();
            lockedDescriptions.clear();
        } catch (Exception e) { e.printStackTrace(); }
    }

    public void resetObjectCounters() {
        Ellipse.resetObjectCounter();
        Line.resetObjectCounter();
        Picture.resetObjectCounter();
        Rectangle.resetObjectCounter();
        StraightLine.resetObjectCounter();
        Text.resetObjectCounter();
    }

    public void updateObject(DrawObject draw_object) {
        properties_.setDrawObject(draw_object);
    }

    public void modAddDrawObject(DrawObject draw_object) {
       if (isInActionList(draw_object.getName())) {
            for(int i = 0; i < objectList.size(); i++) {
                DrawObject obj = (DrawObject)objectList.elementAt(i);
                if(obj.getName().equals(draw_object.getName())) {
                    if(mWin.getDrawingPanel().getLines().remove(obj))
                        mWin.getDrawingPanel().addDrawObject(draw_object);
                    objectList.set(i, draw_object);
                    mWin.getDrawingPanel().updateObjectPointer(draw_object.getPropertyX(),draw_object.getPropertyY());

                    break;
                }
            }
        }

        else {
            objectList.add(draw_object);
            mWin.getDrawingPanel().addDrawObject(draw_object);
            DescriptionHandler dh = new DescriptionHandler(draw_object.getName(), "", "open");
            descriptions.add(dh);
            String text = "";
            String name = draw_object.getName().substring(0, draw_object.getName().indexOf(" "));

            if(name.equals("Text")) {
                Text textObj = (Text)draw_object;
                text = (String)textObj.getPropertyText();
                dh.setObjectDescription(text);
                dh.setObjectStatus("set");
            }
            else {
                list_model_.addElement(draw_object.getName());
            }

            if (text == null || text.length() < 1) actionList.addItem(draw_object.getName());
            else actionList.addItem(draw_object.getName() + " - " + text);
        }
    }

    public void newDescMod(DescriptionHandler newDH) {

        // a DH has been described or deleted...
        if (newDH.getObjectStatus().equals("set")) {
            list_model_.removeElement(newDH.getObjectName());

            if (newDH.getObjectDescription() != null && newDH.getObjectDescription().length() > 0) {
                replaceDescription(newDH);
                deleteActionListItem(newDH.getObjectName());
                actionList.addItem(newDH.getObjectName() + " - " + newDH.getObjectDescription());
            }

        // a DH has been locked
        }
        else if (newDH.getObjectStatus().equals("lock")) {
            if (myLockedObjectStr == null || !myLockedObjectStr.equals(newDH.getObjectName())) {
                list_model_.removeElement(newDH.getObjectName());
            }
        }
        else if (newDH.getObjectStatus().equals("open")) {
            list_model_.addElement(newDH.getObjectName());
        }
        doSelection();
    }

    // Helper function for newDescMod
    public void replaceDescription(DescriptionHandler dH) {
        for(int j = 0; j < descriptions.size(); j++) {
            DescriptionHandler existingDH = (DescriptionHandler)descriptions.elementAt(j);

            if(existingDH.getObjectName().equals(dH.getObjectName())) {
                existingDH.setObjectDescription(dH.getObjectDescription());
                existingDH.setObjectStatus(dH.getObjectStatus());
                return;
            }
        }
    }
}
