/*
 *  Copyright (c) 2002
 *  bestsolution EDV Systemhaus GmbH,
 *  http://www.bestsolution.at
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Header: /usr/bestsolution/cvsroot/java/draw_swf/at/bestsolution/drawswf/DrawingPanel.java,v 1.46 2003/03/14 18:45:40 tom Exp $
 */

package at.bestsolution.drawswf;

import java.io.File;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Vector;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.Toolkit;
import java.awt.Image;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.MediaTracker;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Rectangle2D;

import org.w3c.dom.Element;

import javax.swing.JPanel;
import org.jdom.*;


import at.bestsolution.drawswf.drawobjects.*;

/**
 * @author  heli
 */
public class DrawingPanel extends JPanel implements MouseListener, MouseMotionListener, KeyListener
{
    private BasicStroke stroke_;
    private Color pen_color_;
    private Color fill_color_;
    private LinkedList draw_list_;
    private LinkedList redo_list_;
    private LinkedList paletteList;
    private volatile boolean replay_;
    private Dimension size_;
    private double scale_;
    private int aspect_width_;
    private int aspect_height_;
    private int offset_x_;
    private int offset_y_;
    private Image background_image_;
    private static final Color BACKGROUND_COLOR = Color.white;
    private int drawing_mode_;
    private DrawObject draw_object_;
    private AnimationThread animation_thread_;
    private DrawObjectList drawObjectList;
    private Point drag_start_;

    public String theSaveFile;
    private MainWindow mWin;

    /**
     * Creates a new instance of DrawingPanel
     *
     * @param size the initial size.
     */
    public DrawingPanel(Dimension size, DrawObjectList draw_object_list, MainWindow mW)
    {
        super();

        mWin = mW;
        java.util.Date now = new java.util.Date();
        String path = System.getProperty("user.home")+ File.separator + ".drawswf"+File.separator;
        theSaveFile = path + now.getTime() + "whiteboard.svg";

        animation_thread_ = null;
        drawObjectList = draw_object_list;

        size_ = size;
        aspect_width_ = 5;
        aspect_height_ = 3;
        scale_ = 1.0;

        drawing_mode_ = DrawObjectFactory.MAX_OBJECTS; // this is also the edit mode
        setMinimumSize(size_);
        setPreferredSize(size_);
        setSize(size_);
        setBackground(Color.WHITE);

        stroke_ = new BasicStroke(1.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
        pen_color_ = Color.black;
        fill_color_ = Color.blue;

        draw_list_ = new LinkedList();
        redo_list_ = new LinkedList();
        paletteList = new LinkedList();
        replay_ = false;

        addKeyListener(this);
        addMouseListener(this);
        addMouseMotionListener(this);
        background_image_ = null;

        setFocusable(true);
    }

    // function for removing listeners
    public void startWait() {
        removeKeyListener(this);
        removeMouseListener(this);
        removeMouseMotionListener(this);
    }

    // function for adding listeners
    public void endWait() {
        startWait();
        addKeyListener(this);
        addMouseListener(this);
        addMouseMotionListener(this);
    }

    public int getIndex(String name)  {
        for (int i=0; i<draw_list_.size(); i++) {
            DrawObject temp = (DrawObject) draw_list_.get(i);
            if (temp.getName().equals(name)) {
                return i;
            }
        }
        return -1;
    }

    public int getNewNumber() {
        int max = 1;
        for (int i=0; i<draw_list_.size(); i++) {
            DrawObject temp = (DrawObject) draw_list_.get(i);
            if (temp.getObjectId() >= max) {
            max = temp.getObjectId() + 1;
        }
        }
    return max;
    }

    public DrawObject getObjectByName(String name) {
        for (int i=0; i<draw_list_.size(); i++) {
        DrawObject temp = (DrawObject) draw_list_.get(i);
        if (temp.getName().equals(name)) {
            return temp;
        }
    }
    return null;
    }


    public void loadAnimation(File file, boolean append)
    {
        LinkedList new_list = DrawObjectFactory.loadFile(mWin, file, this);
        int aspect_width = 4;
        int aspect_height = 3;

/*
        if (new_list != null)
        {
            if (append == false)
            {
                //draw_list_.clear();
                //drawObjectList.clearDrawObjects();

                try
                {
                    aspect_width = Integer.parseInt(new_list.get(0).toString()) / 200;
                    aspect_height = Integer.parseInt(new_list.get(1).toString()) / 200;
                }
                catch (NumberFormatException e)
                {
                }

                setCanvasSize(aspect_width, aspect_height);
            }

            for (i = 2; i < new_list.size(); i++)
            {
                draw_list_.add(new_list.get(i));
                drawObjectList.addDrawObject((DrawObject) new_list.get(i));
            }

            repaint();
        }
*/
    }

    public DrawObjectList getDrawObjectList()
    {
        return drawObjectList;
    }

/* Mike Lam */
    public void loadIncomingWhiteboard(Vector svgStrings)
    {
        try
        {
            File newDrawing = new File(theSaveFile);
            PrintWriter out = new PrintWriter(new FileWriter(newDrawing));

            for(int i = 0; i < svgStrings.size(); i++)
            {
                if(i < 2)
                {
                    out.println((String)svgStrings.elementAt(i));
                }

                if(i == 2)
                {
                    out.print("  ");
                    out.println((String)svgStrings.elementAt(i));
                }

                if(i > 2 && (i < svgStrings.size() - 1))
                {
                    out.print("    ");
                    out.println((String)svgStrings.elementAt(i));
                }

                if(i == svgStrings.size() - 1)
                {
                    out.println((String)svgStrings.elementAt(i));
                }
            }

            out.close();

            draw_list_.clear();

            LinkedList new_list = DrawObjectFactory.loadFile(mWin, newDrawing, this);

            for(int i = 2; i < new_list.size(); i++)
            {
                draw_list_.add((DrawObject)new_list.get(i));
                drawObjectList.addDrawObject((DrawObject)new_list.get(i));
            }

            repaint();
        }

        catch ( Exception e )
        {
            System.out.println( e.getMessage() );
        }
    }

    public void addDrawObject(DrawObject dO) {
        draw_list_.add(dO);
        repaint();
    }

    public void addDrawObjectFromPalette(DrawObject drawObject, Element desc)
    {
        addDrawObject(drawObject);
        drawObjectList.addDrawObject(drawObject);

        String name = drawObject.getName();
        String description = desc.getAttribute("description");
        String status = desc.getAttribute("status");

        DescriptionHandler dh = new DescriptionHandler(name, description, status);
        drawObjectList.addDescriptionFromPalette(dh, drawObject);

        mWin.sendModWhiteboard("", drawObject);
    }

    public void saveAnimation(File file)
    {
        DrawObjectFactory.saveFile(draw_list_, file, this, drawObjectList.getAllDescriptionsSVG());
    }

    public void deleteFile() {
        try {
            File savedFile = new File(theSaveFile);
            if (savedFile.exists()) savedFile.delete();
        } catch (Exception e) { e.printStackTrace(); }
    }


    public void setDrawingMode(int drawing_mode)
    {
        drawing_mode_ = drawing_mode;
    }


    public void setBackgroundImage(File image_file)
    {
        if ((image_file == null) || (!image_file.exists()))
        {
            background_image_ = null;
        }
        else
        {
            background_image_ = Toolkit.getDefaultToolkit().getImage(image_file.getPath());
            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(background_image_, 1);
            try
            {
                // wait til image is loaded.
                tracker.waitForID(1);
            }
            catch (InterruptedException e)
            {
                background_image_ = null;
            }
        }

        repaint();
    }


    public void setCanvasSize(int aspect_width, int aspect_height)
    {
        aspect_width_ = aspect_width;
        aspect_height_ = aspect_height;

        repaint();
    }


    public Dimension getCanvasSize()
    {
        return new Dimension(aspect_width_ * 200, aspect_height_ * 200);
    }


    public LinkedList getLines()
    {
        return draw_list_;
    }


    public void clearDrawing()
    {
        while (!draw_list_.isEmpty())
        {
            redo_list_.add(draw_list_.getLast());
            drawObjectList.removeDrawObject((DrawObject) draw_list_.removeLast());
        }

        draw_object_ = null;
        repaint();
    }


    public void redoLastDrawObject()
    {
        if (redo_list_.size() > 0)
        {
            draw_list_.add(redo_list_.getLast());
            drawObjectList.addDrawObject((DrawObject) redo_list_.removeLast());
            repaint();
        }
    }


    public void clearLastDrawObject()
    {
        if (draw_list_.size() > 0)
        {
            if (draw_object_ == draw_list_.getLast())
            {
                draw_object_ = null;
            }

            redo_list_.add(draw_list_.getLast());
            drawObjectList.removeDrawObject((DrawObject) draw_list_.removeLast());
            repaint();
        }
    }


    public void moveDrawingObject(int selected_index, int amount)
    {
        if ( amount != 0 )
        {
            if ( ! ( selected_index + amount >= draw_list_.size() || selected_index + amount < 0 ) )
            {
                Object tmp = draw_list_.get(selected_index);
                Object tmp2 = draw_list_.set(selected_index + amount, tmp);
                draw_list_.set(selected_index, tmp2);
                drawObjectList.moveDrawObject(selected_index, amount);
                repaint();
            }
        }
        else
        {
            drawObjectList.removeDrawObject( (DrawObject) draw_list_.remove(selected_index) );
            repaint();
        }
    }

    public LinkedList getDrawingList()
    {
        return draw_list_;
    }


    public void finishedAnimation()
    {
        animation_thread_ = null;
        replay_ = false;
    }


    public void replay(boolean start)
    {
        if ((start == true) && (animation_thread_ == null))
        {
            replay_ = true;
            animation_thread_ = new AnimationThread(this);
            animation_thread_.start();
        }
        else if (start == false)
        {
            replay_ = false;
        }
    }


    private void pause(long milli)
    {
        try
        {
            Thread.sleep(milli);
        }
        catch (InterruptedException e)
        {
        }
    }


    private int calculateCanvasSizeUnit()
    {
        int width = getWidth();
        int height = getHeight();
        int unit;

        if ((width / aspect_width_) < (height / aspect_height_))
        {
            unit = width / aspect_width_;
        }
        else
        {
            unit = height / aspect_height_;
        }

        return unit;
    }


    public int getCanvasWidth()
    {
        return aspect_width_ * 200;
    }


    public int getCanvasHeight()
    {
        return aspect_height_ * 200;
    }


    private void initCanvas(Graphics2D g)
    {
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        int unit = calculateCanvasSizeUnit();
        int width = unit * aspect_width_;
        int height = unit * aspect_height_;

        offset_x_ = 0;
        offset_y_ = 0;

        g.setColor(getBackground());
        g.fillRect(0, 0, getWidth(), getHeight());

        if (background_image_ == null)
        {
            g.setColor(BACKGROUND_COLOR);
            g.fillRect(offset_x_, offset_y_, width, height);
        }
        else
        {
            g.drawImage(background_image_, offset_x_, offset_y_, width, height, null);
        }

        scale_ = unit / 200.0;

        g.translate(offset_x_, offset_y_);
        g.scale(scale_, scale_);
    }


    protected void paintLines()
    {
        paintLines((Graphics2D) getGraphics());
    }


    public boolean isReplay()
    {
        return replay_;
    }

    public Rectangle2D.Double selectRect = null;
    public double xPt;
    public double yPt;

    private void paintLines(Graphics2D g)
    {
        DrawObject draw_object;

        initCanvas(g);

        for (int count = 0; count < draw_list_.size(); count++)
        {
            draw_object = (DrawObject) draw_list_.get(count);
            draw_object.drawObject(g, this);
        }

        // ********************************** Taras
        if((draw_list_.size() > 0)&&
           (selectRect != null)) {
            g.setColor(Color.red);
            g.fill(selectRect);
            g.draw(selectRect);
        }
    }

    /* Mike Lam */
    public void updateObjectPointer(double xPt,double yPt) {
        selectRect = new Rectangle2D.Double(xPt,yPt,10,10);
    }

    public void paintComponent(Graphics g)
    {
        paintLines((Graphics2D) g);
    }


    public float getPenSize()
    {
        return stroke_.getLineWidth();
    }


    public void setPenSize(float pen_size)
    {
        stroke_ = new BasicStroke(pen_size, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    }


    public Color getPenColor()
    {
        return pen_color_;
    }


    public void setPenColor(Color pen_color)
    {
        pen_color_ = pen_color;
    }


    public Color getFillColor()
    {
        return fill_color_;
    }


    public void setFillColor(Color fill_color)
    {
        fill_color_ = fill_color;
    }


    public void mouseClicked(MouseEvent event)
    {
        if (!mWin.isWaiting()) this.requestFocus();
    }


    public void mouseEntered(MouseEvent event)
    {
    }


    public void mouseExited(MouseEvent event)
    {
    }


    private void selectObject(MouseEvent event) {
        if (!mWin.isWaiting()) {
            DrawObject draw_object;
            DrawObject result = draw_object_;
            Point p = new Point((int) ((event.getX() - offset_x_) / scale_), (int) ((event.getY() - offset_y_) / scale_));

            ListIterator iterator = draw_list_.listIterator();
            while (iterator.hasNext()) {
                draw_object = (DrawObject) iterator.next();

                if (draw_object.contains(p) && (result != draw_object))
                    result = draw_object;
            }

            draw_object_ = result;

            if (draw_object_ != null)
                drawObjectList.setSelectedObject(draw_object_);
        }
    }


    private void dragObject(MouseEvent event) {
        if (!mWin.isWaiting()) {
            if ((drawing_mode_ == DrawObjectFactory.MAX_OBJECTS) && (draw_object_ != null)) {
                Graphics2D g = (Graphics2D) getGraphics();
                g.translate(offset_x_, offset_y_);
                g.scale(scale_, scale_);
                g.setXORMode(Color.white);
                draw_object_.drawObject(g, this);

                double x = ((event.getX() - drag_start_.getX()) / scale_);
                double y = ((event.getY() - drag_start_.getY()) / scale_);
                drag_start_ = event.getPoint();

                draw_object_.move(x, y);
                draw_object_.drawObject(g, this);
            }
        }
    }


    public void mousePressed(MouseEvent event) {
        if (!mWin.isWaiting()) {
            if (drawing_mode_ == DrawObjectFactory.MAX_OBJECTS) {
                drag_start_ = event.getPoint();
                selectObject(event);
            }
            else {
                Graphics2D g = (Graphics2D) getGraphics();
                g.translate(offset_x_, offset_y_);
                g.scale(scale_, scale_);

                draw_object_ = DrawObjectFactory.createObject(mWin, drawing_mode_, pen_color_, fill_color_, stroke_, drawObjectList.getNum());

                if (draw_object_ != null) {
                    int x = (int) ((event.getX() - offset_x_) / scale_);
                    int y = (int) ((event.getY() - offset_y_) / scale_);
                    draw_object_.mousePressed(x, y, g);
                }
            }

            redo_list_.clear();
        }
    }


    public void mouseReleased(MouseEvent event) {
        if (!mWin.isWaiting()) {
            if (drawing_mode_ == DrawObjectFactory.MAX_OBJECTS) {
                if (draw_object_ != null) {
                    drawObjectList.updateObject(draw_object_);
                    updateObjectPointer(draw_object_.getPropertyX(),draw_object_.getPropertyY());
                    mWin.sendModWhiteboard("", draw_object_);
                }

                repaint();
            }

            else {
                Graphics2D g = (Graphics2D) getGraphics();
                g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                g.translate(offset_x_, offset_y_);
                g.scale(scale_, scale_);

                if (draw_object_ != null) {
                    int x = (int) ((event.getX() - offset_x_) / scale_);
                    int y = (int) ((event.getY() - offset_y_) / scale_);

                    if (draw_object_.mouseReleased(x, y, g))
                    {
                        // *******************************************************************
                        // Adding boolean to automatically re-send.... will do in DrawObjectList
                        draw_list_.add(draw_object_);
                        drawObjectList.addDrawObject(draw_object_, true);
                    }
                    else
                        draw_object_ = null;
                }
            }
        }
    }


    public void mouseDragged(MouseEvent event) {
        if (!mWin.isWaiting()) {
            if (drawing_mode_ == DrawObjectFactory.MAX_OBJECTS) {
                dragObject(event);
            }

            else {
                Graphics2D g = (Graphics2D) getGraphics();
                g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
                g.translate(offset_x_, offset_y_);
                g.scale(scale_, scale_);

                if (draw_object_ != null) {
                    int x = (int) ((event.getX() - offset_x_) / scale_);
                    int y = (int) ((event.getY() - offset_y_) / scale_);
                    draw_object_.mouseDragged(x, y, g);
                }
            }
        }
    }


    public void mouseMoved(MouseEvent event){}
    public void keyTyped(KeyEvent event) {}

    public void keyReleased(KeyEvent event) {
        repaint();
        mWin.sendModWhiteboard("", draw_object_);
    }

    public void keyPressed(KeyEvent event) {
        if (!mWin.isWaiting()) {
            selectObjectFromListUsingKeyBoard();

            keyboardAccessibility(event);
        }
    }

    public void keyboardAccessibility(KeyEvent event)
    {
        Graphics2D g = (Graphics2D) getGraphics();
        g.translate(offset_x_, offset_y_);
        g.scale(scale_, scale_);
        draw_object_.keyPressed(g, event);
        drawObjectList.updateObject(draw_object_);

        for(int i = 0; i < draw_list_.size(); i++)
        {
            if(draw_object_.getName().equals(((DrawObject)draw_list_.get(i)).getName()))
                draw_list_.set(i, draw_object_);
        }
    }

    public void selectObjectFromListUsingKeyBoard()
    {
        if (!mWin.isWaiting()) {
            if(drawing_mode_ != DrawObjectFactory.MAX_OBJECTS) {
                String temp = drawObjectList.getSelectedObject();

                if(draw_list_ != null) {
                    ListIterator iterator = draw_list_.listIterator();

                    while (iterator.hasNext()) {
                        draw_object_ = (DrawObject)iterator.next();

                        if(temp.equals(draw_object_.getName()))
                            break;
                    }
                }
            }
        }
    }
}
