/*
 *  Copyright (c) 2002
 *  bestsolution EDV Systemhaus GmbH,
 *  http://www.bestsolution.at
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Header: /usr/bestsolution/cvsroot/java/draw_swf/at/bestsolution/drawswf/FontDialog.java,v 1.12 2003/01/24 01:35:53 tom Exp $
 */

package at.bestsolution.drawswf;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;

import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import javax.swing.text.AttributeSet;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import at.bestsolution.drawswf.fontdialog.OpenList;
import at.bestsolution.drawswf.drawobjects.DrawSWFFont;

/**
 *
 */
public class FontDialog extends JDialog
{
    protected int status_ = JOptionPane.CANCEL_OPTION;
    protected OpenList font_names_;
    protected OpenList font_sizes_;
    protected MutableAttributeSet attributes_;
    protected int effect_;
    private JRadioButton[] effect_buttons_;
    private ChangeListener change_listener_;
    private DrawSWFFont draw_font_;
    
    protected JTextField text_field_;
    private static String[] sizes_ = {"8", "9", "10", "12", "14", "18", "24", "36", "48", "64", "96"};
    
    //----------------------------------------------------------------------------
    /**
     * Constructor for FontDialog
     *
     * @param parent the parent frame needed by all dialogs
     * @param text the text we want to set.
     */
    public FontDialog( JFrame parent )
    {
        super(parent, "Font", true);
        
        effect_ = DrawSWFFont.NO_EFFECT;
        effect_buttons_ = new JRadioButton[3];
        change_listener_ = null;
        draw_font_       = null;
        
        getContentPane().setLayout(new BoxLayout(getContentPane(),BoxLayout.Y_AXIS));
        
        getContentPane().add(createFontSelection());
        getContentPane().add(createFontEffects());
        getContentPane().add(Box.createVerticalStrut(5));
        getContentPane().add(createTextField());
        getContentPane().add(createButtons());

        updatePreview();

        pack();
        Dimension d1 = getSize();
        Dimension d2 = parent.getSize();
        int x = Math.max((d2.width-d1.width)/2, 0);
        int y = Math.max((d2.height-d1.height)/2, 0);
        setBounds(x, y, d1.width, d1.height);
    }
    
    //----------------------------------------------------------------------------
    private JPanel createFontEffects()
    {
        JPanel panel = new JPanel(new GridLayout(1, 3, 10, 5));
        panel.setToolTipText("Effects are only visible in Flash.");
        panel.setBorder(new TitledBorder(new EtchedBorder(), "Effects"));
        
        ButtonGroup group = new ButtonGroup();
        ActionListener listener = new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                effect_ = Integer.parseInt( e.getActionCommand() );
            }
        };
        
        effect_buttons_[0] = new JRadioButton("No effect");
        effect_buttons_[0].setSelected(true);
        effect_buttons_[0].setActionCommand("" + DrawSWFFont.NO_EFFECT);
        panel.add(effect_buttons_[0]);
        group.add(effect_buttons_[0]);
        effect_buttons_[0].addActionListener(listener);
        
        effect_buttons_[1] = new JRadioButton("Type chars");
        effect_buttons_[1].setActionCommand("" + DrawSWFFont.EFFECT_TYPE);
        panel.add(effect_buttons_[1]);
        group.add(effect_buttons_[1]);
        effect_buttons_[1].addActionListener(listener);
        
        effect_buttons_[2] = new JRadioButton("Fade in");
        effect_buttons_[2].setActionCommand("" + DrawSWFFont.EFFECT_FADE_IN);
        panel.add(effect_buttons_[2]);
        group.add(effect_buttons_[2]);
        effect_buttons_[2].addActionListener(listener);
        
        return panel;
    }
    
    //----------------------------------------------------------------------------
    private JPanel createFontSelection()
    {
        JPanel panel = new JPanel(new GridLayout(1, 2, 10, 2));
        panel.setBorder(new TitledBorder(new EtchedBorder(), "Font"));
        
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        font_names_ = new OpenList(ge.getAvailableFontFamilyNames(), "Name:");
        panel.add(font_names_);
        
        font_sizes_ = new OpenList(sizes_, "Size:");
        panel.add(font_sizes_);
        
        ListSelectionListener lsel = new ListSelectionListener()
        {
            public void valueChanged(ListSelectionEvent e)
            {
                updatePreview();
            }
        };

        font_sizes_.setSelected("24");
        font_names_.setSelected(ge.getAvailableFontFamilyNames()[0]);

        font_names_.addListSelectionListener(lsel);
        font_sizes_.addListSelectionListener(lsel);
        
        return panel;
    }
    
    //----------------------------------------------------------------------------
    private JPanel createButtons()
    {
        JPanel panel        = new JPanel(new FlowLayout());
        JPanel inner_panel  = new JPanel(new GridLayout(1, 2, 10, 2));
        JButton button      = new JButton("OK");
        ActionListener action_listener = new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                status_ = JOptionPane.OK_OPTION;
                draw_font_ = new DrawSWFFont( text_field_.getFont(), effect_, text_field_.getText() );
                setVisible(false);
                
                if( change_listener_ != null )
                {
                    change_listener_.stateChanged( new ChangeEvent( this ) );
                }
            }
        };
        button.addActionListener(action_listener);
        inner_panel.add(button);
        
        button = new JButton("Cancel");
        action_listener = new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                status_ = JOptionPane.CANCEL_OPTION;
                setVisible(false);
            }
        };
        button.addActionListener(action_listener);
        inner_panel.add(button);
        panel.add(inner_panel);
        
        return panel;
    }
    
    //----------------------------------------------------------------------------
    private JPanel createTextField()
    {
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(new TitledBorder(new EtchedBorder(), "Text"));
        text_field_ = new JTextField("Preview Font");
        text_field_.setBackground(Color.white);
        text_field_.setForeground(Color.black);
        text_field_.setOpaque(true);
        text_field_.setBorder(new LineBorder(Color.black));
        text_field_.setPreferredSize(new Dimension(120, 40));
        panel.add(text_field_, BorderLayout.CENTER);
        
        return panel;
    }
    
    //----------------------------------------------------------------------------
    public void setAttributes(AttributeSet a)
    {
        attributes_ = new SimpleAttributeSet(a);
        String name = StyleConstants.getFontFamily(a);
        font_names_.setSelected(name);
        int size = StyleConstants.getFontSize(a);
        font_sizes_.setSelectedInt(size);
        
        updatePreview();
    }
    
    //----------------------------------------------------------------------------
    public AttributeSet getAttributes()
    {
        if (attributes_ != null)
        {
            StyleConstants.setFontFamily(attributes_, font_names_.getSelected());
            StyleConstants.setFontSize(attributes_, font_sizes_.getSelectedInt());
        }
        
        return attributes_;
    }
    
    //----------------------------------------------------------------------------
    public int getOption()
    {
        return status_;
    }
    
    //----------------------------------------------------------------------------
    public void setDrawSWFFont( DrawSWFFont font )
    {
        font_names_.setSelected( font.getAWTFont().getFontName() );
        font_sizes_.setSelected( Integer.toString( font.getAWTFont().getSize() ) );
        effect_buttons_[font.getEffect()].setSelected( true );
        draw_font_ = font;
        text_field_.setText( font.getText() );
        updatePreview();
    }
    
    //----------------------------------------------------------------------------
    public DrawSWFFont getDrawSWFFont()
    {
        return draw_font_;
    }
    
    public void setTextInputEnabled( boolean yes )
    {
        text_field_.setEditable(yes);
    }
    
//    //----------------------------------------------------------------------------
//    public String getText()
//    {
//        return text_field_.getText();
//    }
//    
//    //----------------------------------------------------------------------------
//    public void setText( String text )
//    {
//        text_field_.setText( text );
//    }
    
    //----------------------------------------------------------------------------
    public void addChangeListener(ChangeListener change_listener)
    {
        // very dirty... should be a vector
        change_listener_ = change_listener;
    }
    
    //----------------------------------------------------------------------------
    protected void updatePreview()
    {
        String name = font_names_.getSelected();
        int size = font_sizes_.getSelectedInt();
        
        if (size > 0)
        {
            Font font = new Font( name, Font.PLAIN, size );
            text_field_.setFont( font );
            text_field_.repaint();
        }
        
        if( change_listener_ != null )
        {
            draw_font_ = new DrawSWFFont( text_field_.getFont(), effect_, text_field_.getText() );
            change_listener_.stateChanged( new ChangeEvent( this ) );
        }
    }
}
