/*
 *  Copyright (c) 2002
 *  bestsolution EDV Systemhaus GmbH,
 *  http://www.bestsolution.at
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Header: /usr/bestsolution/cvsroot/java/draw_swf/at/bestsolution/drawswf/MainWindow.java,v 1.42 2003/03/07 18:47:46 tom Exp $
 *
 * Modifications by:  Mike Lam
 */

package at.bestsolution.drawswf;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Vector;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Timer;
import java.util.TimerTask;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Image;
import java.awt.Color;
import java.awt.Cursor;

import javax.swing.JFrame;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.BorderFactory;

import at.bestsolution.drawswf.util.DrawSWFConfig;

import com.incors.plaf.kunststoff.KunststoffLookAndFeel;
import com.incors.plaf.kunststoff.KunststoffTheme;

import ca.utoronto.atrc.whiteboard.*;
import at.bestsolution.drawswf.drawobjects.*;
import at.bestsolution.drawswf.DrawObjectList;
import at.bestsolution.drawswf.toolbar.DrawToolsToolbar;

/**
 * The main window of the application.
 */
public class MainWindow extends JPanel {
    public DrawingPanel drawingPanel = null;
    private static DrawObjectList draw_object_list_;
    private static final String IMAGE_PATH = "at/bestsolution/drawswf/images/";
    private static DrawSWFConfig config_;
    private static ResourceBundle international_;
    public static MainWindow MAIN_WINDOW = null;
    private DrawToolsToolbar tool_bar_;
    private ArrayList plugins_;

    public Vector svgAllWhiteboardObjects = null;
    public Vector svgElements = new Vector();

    // taras additions
    public ACommunicator aComm;
    public String jid;
    public static JFrame theFrame;

    public boolean isGroup = false;
    protected boolean isWaiting = false;

    public String myUserName;

    private Timer messageSentTimer;

    public MainWindow(ACommunicator aC, String j, JFrame tF, boolean iG)
    {
        super();
        aComm = aC;
        jid = j;
        myUserName = aComm.toolKit.jidUsername(aComm.myJID);
        theFrame = tF;
        MAIN_WINDOW = this;
        plugins_ = null;
        isGroup = iG;
    }

    public boolean isWaiting() {
        if (isGroup != false)
            return isWaiting;
        return false;
    }

    public void setWaiting(boolean value) {
        if (isGroup != false)
            isWaiting = value;
    }

    public void killIt() {
        drawingPanel.deleteFile();
    }

    public ArrayList getPlugins()
    {
        return plugins_;
    }

    public static String getImagePath()
    {
        return IMAGE_PATH;
    }

    public static DrawSWFConfig getConfiguration()
    {
        return config_;
    }

    public static ResourceBundle getI18n()
    {
        return international_;
    }

    public DrawingPanel getDrawingPanel()
    {
        return drawingPanel;
    }

    public DrawToolsToolbar getDrawToolbar()
    {
        return tool_bar_;
    }

    public DrawObjectList getDrawObjectList() {
        return draw_object_list_;
    }

    public void drawIt()
    {
        setI18n();

        setUI();
        setIcon();
        setLayout(new BorderLayout());

        JPanel descriptionPanel = new JPanel();

        // descriptionPanel = panel at the top which contains the whiteboard
        descriptionPanel.setLayout(new BorderLayout());
        Dimension size = new Dimension( 350, 200 );

        draw_object_list_ = new DrawObjectList(this);
        drawingPanel = new DrawingPanel( size, draw_object_list_, this);
        //tool_bar_ = new DrawToolBar( draw_object_list_, this);
        tool_bar_ = new DrawToolsToolbar(this, draw_object_list_);

        descriptionPanel.add(tool_bar_, "North");
        descriptionPanel.add(drawingPanel, "Center");
        descriptionPanel.add(draw_object_list_, "South");
        descriptionPanel.setBorder(BorderFactory.createTitledBorder("Whiteboard"));
        draw_object_list_.setFocusable(true);

        add(descriptionPanel, "North");
        add(draw_object_list_.addEditDescriptionsPanel(), "South");

        PluginLoader loader = new PluginLoader(this);
        plugins_ = loader.loadPlugins();
   }

   private void setI18n()
   {
        DrawSWFConfig config = DrawSWFConfig.getInstance();

        if(config.getProperty("pluginpath") == null)
        {
            String path = System.getProperty("user.home")+ File.separator + ".drawswf"+File.separator+"plugin";
            config.setProperty("pluginpath", path);

            try
            {
                File dir = new File(path);

                if( ! dir.exists() )
                {
                    dir.mkdirs();
                }

                config.save();
            }
            catch( FileNotFoundException e )
            {
                e.printStackTrace();
            }
            catch(IOException e)
            {
                e.printStackTrace();
            }
        }

        Locale locale = new Locale(config.getProperty("language") );

        international_ = ResourceBundle.getBundle("DrawSWFBundle", locale );
    }

    private void setIcon()
    {
        Image icon;
        URL icon_url;

        icon_url = getClass().getClassLoader().getResource("at/bestsolution/drawswf/images/logo_icon.png");
        icon = Toolkit.getDefaultToolkit().getImage( icon_url );
        theFrame.setIconImage( icon );
    }

    private void setUI()
    {
        try
        {
            KunststoffLookAndFeel kunststoffLnF = new KunststoffLookAndFeel();
            KunststoffLookAndFeel.setCurrentTheme( new KunststoffTheme() );
            UIManager.setLookAndFeel(kunststoffLnF);
        }
        catch (UnsupportedLookAndFeelException ex)
        {
            // handle exception or not, whatever you prefer
        }
        // this line needs to be implemented in order to make JWS work properly
        UIManager.getLookAndFeelDefaults().put("ClassLoader", getClass().getClassLoader());
    }


    private void setFrameToCenter()
    {
        Dimension screen_size = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frame_size = getSize();
        if (frame_size.height > screen_size.height)
        {
            frame_size.height = screen_size.height;
        }
        if (frame_size.width > screen_size.width)
        {
            frame_size.width = screen_size.width;
        }

        setLocation((screen_size.width - frame_size.width) / 2, (screen_size.height - frame_size.height) / 2);

        int new_x = (screen_size.width - frame_size.width) / 2 - draw_object_list_.getWidth();

        if (new_x < 0)
        {
            new_x = 0;
        }

        draw_object_list_.setLocation(new_x, (screen_size.height - frame_size.height) / 2);
    }

    public Vector getAllWhiteboardObjects() {
        DrawObject drawObject;
        svgAllWhiteboardObjects = new Vector();

        for(int i = 0; i < draw_object_list_.objectList.size(); i++) {
            drawObject = (DrawObject) draw_object_list_.objectList.elementAt((int) i);
            svgAllWhiteboardObjects.add( "    " + drawObject.toSVG( i*250 ));
        }
        return svgAllWhiteboardObjects;
    }

    public void sendWhiteboard() {
        sendWhiteboard("", "");
    }

    public void sendWhiteboard(String message) {
        sendWhiteboard(message, "");
    }

    class SetToFalse extends TimerTask {
        public void run() {
            stopMessageSentWait();
            messageSentTimer.cancel(); //Terminate the timer thread
        }
    }

    // In group chat, pause everything until we get the message back
    public void startMessageSentWait() {
        System.out.println("startMessageSentWait");
        if (isGroup) {
            setWaiting(true);
            drawingPanel.startWait();
            //drawingPanel.noReceivedMessage();
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            if (messageSentTimer != null) {
                messageSentTimer.cancel();
                messageSentTimer = null;
            }
            messageSentTimer = new Timer();
            int seconds = 10;
            messageSentTimer.schedule(new SetToFalse(), seconds*1000);
        }
    }

    public void stopMessageSentWait() {
        setWaiting(false);
        drawingPanel.endWait();
        //drawingPanel.hasReceivedMessage();
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    public void sendWhiteboard(String message, String toWhom) {
        startMessageSentWait();
        aComm.sendWhiteboardMessage(jid, "full", getAllWhiteboardObjects(), null, draw_object_list_.getAllDescriptionsSVG(), message, toWhom);
    }

    public void sendModWhiteboardRM(String message, String rmName) {
        startMessageSentWait();
        String theItem = "<rm>"+rmName+"</rm>";
        Vector theRMStuff = new Vector();
        theRMStuff.add(theItem);
        aComm.sendWhiteboardMessage(jid, "mod", null, theRMStuff, null, message);
    }

    public void sendModWhiteboard(String message, DrawObject dO) {
        System.out.println("sendModWhiteboard()");
        startMessageSentWait();
        String theItem = dO.toSVG(0);
        Vector theSVGStuff = new Vector();
        theSVGStuff.add(theItem);
        aComm.sendWhiteboardMessage(jid, "mod", theSVGStuff, null, null, message);
    }

    public void sendModWhiteboardDesc(String message, DescriptionHandler dH) {
        startMessageSentWait();
        Vector theDescObjects = new Vector();
        theDescObjects.add(dH.getXMLString());
        aComm.sendWhiteboardMessage(jid, "mod", null, null, theDescObjects, message);
    }


    public void getWhiteboard() {
        aComm.sendWhiteboardRequest(jid);
    }

    // GROUPCHAT ONLY!! Activated when a message from oneself is recieved
    public void incomingWhiteboardMessageFromMyself(String sender, String type) {
        System.out.println("Incoming From Me!!!!!!!!!!!!!!!!!");
        stopMessageSentWait();
    }

    public void incomingWhiteboardMessage(String sender, String type, Vector svgString, Vector svgRMString, Vector descObjects, String toWhom)
    {
        //System.out.println("Got a message... type="+type+", toWhom="+toWhom);
        System.out.println("Incoming From Someone else - "+sender+"!!!!!!!!!!!!!!!!!");

        if(type.equals("req")) {
            sendWhiteboard("", sender);

        } else if (type.equals("full") && (((toWhom == null) || (toWhom.length() < 1))|| (toWhom.equalsIgnoreCase(myUserName)))) {
            svgElements.clear();
            String header1 = "<?xml version=\"1.0\" encoding= \"UTF-8\"?>";
            String header2 = "<svg width=\"1000\" height=\"600\" xmlns:drawswf=\"http://drawswf.sf.net/2002/drawswf\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">";
            String header3 = "  <title>Made with DrawSWF 1.2 (http://drawswf.sf.net)</title>";
            String header4 = "</svg>";

            svgElements.add(header1);
            svgElements.add(header2);
            svgElements.add(header3);

            for(int i = 0; i < svgString.size(); i++)
            {
                svgElements.add((String)svgString.elementAt(i));
            }

            svgElements.add(header4);
            draw_object_list_.clearDrawObjects();
            drawingPanel.loadIncomingWhiteboard(svgElements);
            draw_object_list_.loadDescriptions(descObjects);

        //System.out.println("Got a FULL message!");

        } else if (type.equals("mod")) {

            if (svgString != null && svgString.size() > 0) {
                for (int i=0; i<svgString.size(); i++) {
                    DrawObject dO = DrawObjectFactory.buildObjectFromXML((String)svgString.elementAt(i), this);
                    //System.out.println("New Element: "+dO.getName());
                    draw_object_list_.modAddDrawObject(dO);
                }
            }

            if (svgRMString != null && svgRMString.size() > 0) {
                String theRMName2 = (String) svgRMString.elementAt(0);
                theRMName2 = DrawObjectFactory.getRMName(theRMName2);
                //System.out.println("Want to delete "+theRMName2);
                draw_object_list_.modDeleteObject(theRMName2);
            }

            if (descObjects != null && descObjects.size() > 0) {
                String newDesc = (String) descObjects.elementAt(0);
                DescriptionHandler dH = new DescriptionHandler(newDesc);
                //System.out.println("New desc: "+newDesc);
                draw_object_list_.newDescMod(dH);
            }
        }
    }
}
