/*
 *  Copyright (c) 2002
 *  bestsolution EDV Systemhaus GmbH,
 *  http://www.bestsolution.at
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Header: /usr/bestsolution/cvsroot/java/draw_swf/at/bestsolution/drawswf/drawobjects/DrawObject.java,v 1.27 2003/01/24 01:35:53 tom Exp $
 */

/*
 * DrawObject.java
 *
 * Created on 21. Mai 2002, 22:41
 */

package at.bestsolution.drawswf.drawobjects;
import java.awt.Color;
import java.awt.event.KeyEvent;
import java.awt.Graphics2D;
import java.awt.BasicStroke;
import java.awt.geom.Point2D;
import com.anotherbigidea.flash.movie.Movie;
import org.w3c.dom.Element;
import at.bestsolution.drawswf.DrawingPanel;
import at.bestsolution.drawswf.MainWindow;
import at.bestsolution.drawswf.DrawObjectList;

/**
 * An abstract class which represents the parent
 * of all draw objects.
 * @author tom
 */
public abstract class DrawObject
{
    /** the stroke used to draw */
    protected BasicStroke stroke_;
    /** the color of lines */
    protected Color pen_color_;
    /** the fill color */
    protected Color fill_color_;
    /** this is the delay in milliseconds for animation */
    protected static final long ANIMATION_DELAY = 5;
    /** name of the draw object */
    protected String name_;

    private MainWindow mWin;

    protected String sender = "";

    //----------------------------------------------------------------------------
    /**
     * Constructor of DrawObject
     *
     * @param name the name of the drawobject
     */
    public DrawObject( String name, MainWindow mW )
    {
        mWin = mW;
        name_ = name;
    sender = mWin.myUserName;
    }

    public void setSender(String s) { sender = s; }
    public String getSender() { return sender; }

    public abstract double getPropertyY();
    public abstract double getPropertyX();

    public abstract int getObjectId();
    public abstract void setObjectId();
    public abstract void setObjectId(int ID);

    //----------------------------------------------------------------------------
    /**
     * This method is called to draw the object onto the
     * drawing panel.
     *
     * @param g the graphics context.
     * @param panel to test isReplay.
     */
    public abstract void drawObject(Graphics2D g, DrawingPanel panel);

    //----------------------------------------------------------------------------
    /**
     * This method is called to draw the object into the
     * flash movie.
     *
     * @param movie the movie to add the drawing.
     * @param layer of the actual draw object.
     * @param the speed to draw the animation.
     */
    public abstract void drawObject(Movie movie, int layer, int speed);

    //----------------------------------------------------------------------------
    /**
     * This is called by the DrawingPanel when the mouse button is pressed.
     *
     * @param x the x position of the mouse pointer
     * @param y the y position of the mouse pointer
     * @param g the graphics context to draw to.
     * @see at.bestsolution.drawswf.DrawingPanel
     */
    public abstract void mousePressed(int x, int y, Graphics2D g);

    //----------------------------------------------------------------------------
    /**
     * This is called by the DrawingPanel when the mouse is dragged.
     *
     * @param x the x position of the mouse pointer
     * @param y the y position of the mouse pointer
     * @param g the graphics context to draw to.
     * @see at.bestsolution.drawswf.DrawingPanel
     */
    public abstract void mouseDragged(int x, int y, Graphics2D g);

    //----------------------------------------------------------------------------
    /**
     * This is called by the DrawingPanel when the mouse button is released.
     *
     * @param x the x position of the mouse pointer
     * @param y the y position of the mouse pointer
     * @param g the graphics context to draw to.
     * @see at.bestsolution.drawswf.DrawingPanel
     */
    public abstract boolean mouseReleased(int x, int y, Graphics2D g);

    //----------------------------------------------------------------------------
    public abstract String toSVG(long time);

    //----------------------------------------------------------------------------
    protected abstract void createAWTObject( Element xml_node, double scale );

    //----------------------------------------------------------------------------
    /**
     * Set the color of the pen.
     *
     * @param color the new color.
     */
    public void setColor(Color color)
    {
        pen_color_ = color;
    }

    //----------------------------------------------------------------------------
    /**
     * Set the fill color.
     *
     * @param color the new color.
     */
    public void setFillColor(Color color)
    {
        fill_color_ = color;
    }

    //----------------------------------------------------------------------------
    /**
     * Set the stroke of lines.
     *
     * @param stroke the new stroke.
     */
    public void setStroke(BasicStroke stroke)
    {
        stroke_ = stroke;
    }

    //----------------------------------------------------------------------------
    /**
     * Wait a given delay.
     *
     * @param milli the delay in milliseconds.
     */
    protected void pause(long milli)
    {
        try
        {
            Thread.sleep(milli);
            Thread.yield();
        }
        catch (InterruptedException e)
        {
        }
    }

    //----------------------------------------------------------------------------
    private Color parseSVGColor( String color, String alpha_text )
    {
        int rgb;
        int red   = 0;
        int green = 0;
        int blue  = 0;
        int alpha = 0;

        try
        {
            rgb   = Integer.decode( color ).intValue();

            red   = (rgb >>> 16) & 0xff;
            green = (rgb >>>  8) & 0xff;
            blue  =  rgb         & 0xff;

            alpha = (int) (Double.parseDouble(alpha_text) * 255.0);
        }
        catch (NumberFormatException e)
        {
        }

        Color parsed_color = new Color( red, green, blue, alpha );

        return parsed_color;
    }

    //----------------------------------------------------------------------------
    private String encodeColor(Color color)
    {
        String encoded = Integer.toHexString( 0xffffff & color.getRGB() );

        if (encoded.length() < 6)
        {
            // fill string with zeros so it has always length of 6
            encoded = "000000".substring( encoded.length() ) + encoded;
        }

        return encoded;
    }

    //----------------------------------------------------------------------------
    protected String toSVGFillColor()
    {
        return " fill=\"#" + encodeColor(fill_color_)
                + "\" fill-opacity=\"" + fill_color_.getAlpha()/255.0 + "\"";
    }

    //----------------------------------------------------------------------------
    protected String toSVGStroke()
    {
        return " stroke=\"#" +  encodeColor(pen_color_)
                + "\" stroke-width=\"" + stroke_.getLineWidth()
                + "\" stroke-opacity=\"" + pen_color_.getAlpha()/255.0
                + "\" stroke-linecap=\"round\"";
    }

    //----------------------------------------------------------------------------
    protected void setAWTFillColor( Element svg_element )
    {
        fill_color_ = parseSVGColor(svg_element.getAttribute("fill"), svg_element.getAttribute("fill-opacity") );
    }

    //----------------------------------------------------------------------------
    protected void setAWTStroke( Element svg_element, double scale )
    {
        pen_color_ = parseSVGColor( svg_element.getAttribute("stroke"), svg_element.getAttribute("stroke-opacity") );
        stroke_ = new BasicStroke( (float) scale * Float.parseFloat( svg_element.getAttribute("stroke-width") ) , BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
    }

    //----------------------------------------------------------------------------
    protected void waitMovie( Movie movie, int speed )
    {
        // sleep in flash...
        for (int count = 0; count < 12 / speed; count++ )
        {
            movie.appendFrame();
        }
    }

    //----------------------------------------------------------------------------
    public abstract String getName();

        //return name_;


    //----------------------------------------------------------------------------
    protected void repaint()
    {
        mWin.drawingPanel.repaint();
    }

    //----------------------------------------------------------------------------
    public abstract boolean contains(Point2D p);

    //----------------------------------------------------------------------------
    public abstract void move(double x, double y);

    public abstract void keyPressed(Graphics2D g, KeyEvent e);
}
