/*
 *  Copyright (c) 2002
 *  bestsolution EDV Systemhaus GmbH,
 *  http://www.bestsolution.at
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Header: /usr/bestsolution/cvsroot/java/draw_swf/at/bestsolution/drawswf/drawobjects/DrawObjectFactory.java,v 1.17 2003/01/24 01:35:53 tom Exp $
 */

package at.bestsolution.drawswf.drawobjects;

import java.awt.Color;
import java.awt.BasicStroke;
import java.util.LinkedList;
import java.util.Vector;

import java.io.File;
import java.io.PrintWriter;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;

import org.apache.xpath.XPathAPI;
import org.xml.sax.SAXException;

import at.bestsolution.drawswf.DrawingPanel;
import at.bestsolution.drawswf.MainWindow;
import at.bestsolution.drawswf.DrawObjectList;
import at.bestsolution.drawswf.DescriptionHandler;

/**
 *
 * @author  tom
 * @modifications Mike Lam, Taras Kowaliw
 */
public class DrawObjectFactory
{
    public static final int LINE          = 0;
    public static final int STRAIGHT_LINE = 1;
    public static final int RECTANGLE     = 2;
    public static final int ELLIPSE       = 3;
    public static final int TEXT          = 4;
    public static final int PICTURE       = 5;
    public static final int MAX_OBJECTS   = 6;

    protected static final String SVG_LINE          = "polyline";
    protected static final String SVG_STRAIGHT_LINE = "line";
    protected static final String SVG_RECTANGLE     = "rect";
    protected static final String SVG_ELLIPSE       = "ellipse";
    protected static final String SVG_TEXT          = "text";
    protected static final String SVG_PICTURE       = "image";

    private static final String FORMAT_DEFINITION = "Made with DrawSWF 1.2 (http://drawswf.sf.net)";

    private static boolean scale_;

    public static Vector svgAllWhiteboardObjects = new Vector();

    public static DrawObject createObject(MainWindow mW, int drawing_mode, Color pen_color, Color fill_color, BasicStroke stroke, int which)
    {
        DrawObject result = null;

        switch( drawing_mode )
        {
            case LINE:
                result = new Line(mW);
                break;
            case RECTANGLE:
                result = new Rectangle(mW);
                break;
            case ELLIPSE:
                result = new Ellipse(mW);
                break;
            case STRAIGHT_LINE:
                result = new StraightLine(mW);
                break;
            case TEXT:
                result = new Text(mW);
                break;
            case PICTURE:
                result = new Picture(mW);
                break;
            default:
                result = null;
                break;
        }

        result.setColor( pen_color );
        result.setFillColor( fill_color );
        result.setStroke( stroke );
        //result.setObjectId(which);

        return result;
    }

    //----------------------------------------------------------------------------
    public static DrawObject createObject(MainWindow mW, Element xml_node )
    {
        double scale = 1.0;
        DrawObject result = null;
        String tag_name = xml_node.getTagName();

        if( tag_name.equals(SVG_LINE) )
        {
            result = new Line(mW);
        }
        else if( tag_name.equals(SVG_RECTANGLE) )
        {
            result = new Rectangle(mW);
        }
        else if( tag_name.equals(SVG_ELLIPSE) )
        {
            result = new Ellipse(mW);
        }
        else if( tag_name.equals(SVG_STRAIGHT_LINE) )
        {
            result = new StraightLine(mW);
        }
        else if( tag_name.equals(SVG_TEXT) )
        {
            result = new Text(mW);
        }
        else if( tag_name.equals(SVG_PICTURE) )
        {
            result = new Picture(mW);
        }

        if( result != null )
        {
            if (scale_ == true)
            {
                scale = 0.05;
            }
            result.createAWTObject( xml_node, scale );
        }

        return result;

    }

    public static String getRMName(String xmlStr) {
        Document document = null;
        Element theElement = null;
        String rmName = null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            java.io.StringReader sR = new java.io.StringReader(xmlStr);
            org.xml.sax.InputSource iS = new org.xml.sax.InputSource(sR);
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(iS);

            theElement = document.getDocumentElement();
            rmName = theElement.getTagName();
            Node textChild = theElement.getFirstChild();

            if (textChild != null) rmName = textChild.getNodeValue();
        } catch (Exception e) { e.printStackTrace(); }

       return rmName;
    }


    public static DrawObject buildObjectFromXML(String xmlStr, MainWindow mW) {
        DrawObject dO = null;
        Document document = null;
        Element theElement = null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            java.io.StringReader sR = new java.io.StringReader(xmlStr);
            org.xml.sax.InputSource iS = new org.xml.sax.InputSource(sR);
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(iS);

            theElement = document.getDocumentElement();
            Element theEl = (Element) theElement;

            dO = DrawObjectFactory.createObject(mW,  theEl );
            if (dO != null) {

                Attr attributeID = (Attr) theEl.getAttributeNode("id");
                int shapeID = Integer.parseInt(attributeID.getValue());
                dO.setObjectId(shapeID);

                Attr senderAtt = (Attr) theEl.getAttributeNode("sender");
                if (senderAtt != null) dO.setSender(senderAtt.getValue());


        /*
        Attr attributeID = (Attr)((Element)theElement).getAttributeNode("id");
                int shapeID = Integer.parseInt(attributeID.getValue());
                dO.setObjectId(shapeID);

        Element senderEl = (Element) theElement.getAttributeNode("sender");
        if (senderEl != null) {
            Attr senderAtt = (Attr) senderEl;
                    if (senderAtt != null) dO.setSender(senderAtt.getValue());
        }   */
            }
       } catch (Exception e) { e.printStackTrace(); }

       return dO;
    }

    //----------------------------------------------------------------------------
    public static LinkedList loadFile(MainWindow mW, File file, DrawingPanel panel )
    {
        LinkedList list     = null;
        NodeList svg_shapes = null;
        NodeList descriptions = null;
        Element root;
        Element g_node;
        Element title;
        Element desc_list;
        Document document;
        DrawObject draw_object;
        Element descElement;

        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setNamespaceAware(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(file.toString());

            root = document.getDocumentElement();
            title = (Element) document.getElementsByTagName( "title" ).item(0);

            if (document.getElementsByTagName( "g" ).getLength() == 0)
            {
                svg_shapes = XPathAPI.selectNodeList( root, "*" );
                scale_ = false;
            }
            else
            {
                svg_shapes = XPathAPI.selectNodeList( document.getElementsByTagName( "g" ).item(0), "*" );
                scale_ = true;
            }

            descriptions = document.getElementsByTagName("desc");

            if ( (title == null) || ! FORMAT_DEFINITION.equals(title.getFirstChild().getNodeValue()) )
            {
                JOptionPane.showMessageDialog( panel, "Wrong or old file format", "Error", JOptionPane.ERROR_MESSAGE );
            }
            else if ( (svg_shapes != null) && (title.getFirstChild() != null) )
            {
                //list = new LinkedList();
                //list.add( root.getAttributeNS(null, "width") );
                //list.add( root.getAttributeNS(null, "height") );

                //panel.getLines().clear();
                //panel.getDrawObjectList().clearDrawObjects();

                int counter = 0;

                for (int i = 0; i < svg_shapes.getLength(); i++)
                {
                    draw_object = DrawObjectFactory.createObject(mW,(Element)svg_shapes.item(i));

                    if (draw_object != null)
                    {
                        descElement = (Element)descriptions.item(counter);

                        if(descElement != null)
                            panel.addDrawObjectFromPalette(draw_object, descElement);

                        counter++;

                        Element theEl = (Element) svg_shapes.item(i);

                        Attr senderAtt = (Attr) theEl.getAttributeNode("sender");
                        if (senderAtt != null) draw_object.setSender(senderAtt.getValue());
                    }
                }
            }
        }
        catch (FactoryConfigurationError e)
        {
            e.printStackTrace();
        }
        catch (ParserConfigurationException e)
        {
            e.printStackTrace();
        }
        catch (SAXException e)
        {
            JOptionPane.showMessageDialog( panel, "Wrong or old file format", "Error", JOptionPane.ERROR_MESSAGE );
            e.printStackTrace();
        }
        catch (IOException e)
        {
            JOptionPane.showMessageDialog( panel, "Could not open file", "Error", JOptionPane.ERROR_MESSAGE );
            e.printStackTrace();
        }
        catch( TransformerException e )
        {
            JOptionPane.showMessageDialog( panel, "Wrong or old file format", "Error", JOptionPane.ERROR_MESSAGE );
            e.printStackTrace();
        }

        return list;
    }

    //----------------------------------------------------------------------------
    public static boolean saveFile( LinkedList draw_list, File target_file, DrawingPanel drawing_panel, Vector descriptionsSVG )
    {
        boolean result = false;
        DrawObject draw_object;

        try
        {
            PrintWriter out = new PrintWriter( new BufferedOutputStream( new FileOutputStream( target_file ) ) );

            out.println("<?xml version=\"1.0\" encoding= \"UTF-8\"?>");
            out.print("<svg width=\"" + drawing_panel.getCanvasWidth() + "\" height=\"" + drawing_panel.getCanvasHeight() );
            out.println( "\" xmlns:drawswf=\"http://drawswf.sf.net/2002/drawswf\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">");
            out.println("  <title>" + FORMAT_DEFINITION + "</title>" );

            for( long i = 0; i < draw_list.size(); i++ )
            {
                draw_object = (DrawObject) draw_list.get((int) i);
                out.println( "    " + draw_object.toSVG( i*250 ) );
            }

            out.println("");

            for(int j = 0; j < descriptionsSVG.size(); j++)
            {
                out.println("    "+(String)descriptionsSVG.elementAt(j));
            }

            out.println("</svg>");
            out.close();

            result = true;
        }
        catch ( FileNotFoundException e )
        {
            System.out.println( e.getMessage() );
        }

        return result;
    }
}
