/*
 *  Copyright (c) 2002
 *  bestsolution EDV Systemhaus GmbH,
 *  http://www.bestsolution.at
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Header: /usr/bestsolution/cvsroot/java/draw_swf/at/bestsolution/drawswf/drawobjects/Ellipse.java,v 1.30 2003/01/24 01:35:53 tom Exp $
 */

/*
 * Line.java
 *
 * Created on 21. Mai 2002, 22:57
 */

package at.bestsolution.drawswf.drawobjects;

import at.bestsolution.drawswf.DrawingPanel;
import at.bestsolution.drawswf.MainWindow;

import com.anotherbigidea.flash.movie.Frame;
import com.anotherbigidea.flash.movie.Instance;
import com.anotherbigidea.flash.movie.Movie;
import com.anotherbigidea.flash.movie.Shape;
import com.anotherbigidea.flash.structs.AlphaColor;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import javax.swing.JTable;
import javax.swing.table.TableModel;

import java.awt.event.KeyEvent;

import org.w3c.dom.Element;

/**
 *
 * @author  tom
 */
public class Ellipse extends DrawObject
{
    private Point2D start_point_;
    private Ellipse2D ellipse;
    private static int instance_counter = 1;
    private TableModel model_;
    private JTable options_table_;
    private MainWindow mWin;
    private int objectID;

    //----------------------------------------------------------------------------
    /** Creates a new instance of Line */
    public Ellipse(MainWindow mW)
    {
        super( "Ellipse ", mW);
        instance_counter++;
        setObjectId();
        mWin = mW;
        ellipse = null;
        start_point_ = null;
    }

    public String getName() {
        return "Ellipse " + getObjectId();
    }

    public static void resetObjectCounter()
    {
        instance_counter = 1;
    }

    public int getObjectId() {
        return objectID;
    }

    public void setObjectId() {
        objectID = instance_counter - 1;
    }

    public void setObjectId(int ID) {
        objectID = ID;
    }

    //----------------------------------------------------------------------------
    public void drawObject(Movie movie, int layer, int speed)
    {
        Frame frame = movie.appendFrame();
        Shape shape = new Shape();
        AlphaColor line_color = new AlphaColor( pen_color_.getRed(), pen_color_.getGreen(), pen_color_.getBlue(), pen_color_.getAlpha() );
        AlphaColor fill_color = new AlphaColor( fill_color_.getRed(), fill_color_.getGreen(), fill_color_.getBlue(), fill_color_.getAlpha() );

        shape.defineFillStyle(fill_color);
        shape.defineLineStyle((stroke_.getLineWidth()), line_color);
        shape.setLineStyle(1);
        shape.setRightFillStyle(1);

        shape.drawAWTPathIterator(ellipse.getPathIterator(null));

        Instance instance = frame.placeSymbol(shape, 0, 0);

        // sleep in flash...
        for (int count = 0; count < 5; count++ )
        {
            movie.appendFrame();
        }
    }

    //----------------------------------------------------------------------------
    public void drawObject(Graphics2D g, DrawingPanel panel)
    {
        g.setStroke(stroke_);
        g.setColor(fill_color_);
        g.fill( ellipse );
        g.setColor(pen_color_);
        g.draw( ellipse );
        if (panel.isReplay() == true)
        {
            pause(100*ANIMATION_DELAY);
        }
    }

    //----------------------------------------------------------------------------
    private java.awt.geom.Ellipse2D.Double newEllipse2D(double end_x, double end_y)
    {
        double ex;
        double ey;
        double width;
        double height;
        double x = start_point_.getX();
        double y = start_point_.getY();

        if (x < end_x)
        {
            ex = x;
            width = end_x - x;
        }
        else
        {
            ex = end_x;
            width = x - end_x;
        }

        if (y < end_y)
        {
            ey = y;
            height = end_y - y;
        }
        else
        {
            ey = end_y;
            height = y - end_y;
        }

        return new java.awt.geom.Ellipse2D.Double(ex, ey, width, height);
    }

    //----------------------------------------------------------------------------
    public void mouseDragged(int x, int y, Graphics2D g)
    {
        g.setXORMode(Color.white);

        g.setColor(pen_color_);
        g.setStroke(stroke_);
        g.draw( ellipse );
        ellipse = newEllipse2D(x, y);
        g.draw( ellipse );
    }

    //----------------------------------------------------------------------------
    public void mousePressed(int x, int y, Graphics2D g)
    {
        g.setColor(pen_color_);
        g.setStroke(stroke_);
        start_point_ = new Point2D.Double(x, y);
        ellipse = newEllipse2D(x, y);
        g.draw( ellipse );
    }

    public void keyPressed(Graphics2D g, KeyEvent e)
    {
        /* The following are for resizing the ellipse */
        if((e.getModifiers() == KeyEvent.CTRL_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_UP))
        {
            ellipse.setFrame(ellipse.getX(), ellipse.getY(), ellipse.getWidth(), ellipse.getHeight()+10);
            g.draw(ellipse);
        }

        if((e.getModifiers() == KeyEvent.CTRL_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_DOWN))
        {
            ellipse.setFrame(ellipse.getX(), ellipse.getY(), ellipse.getWidth(), ellipse.getHeight()-10);
            g.draw(ellipse);
        }

        if((e.getModifiers() == KeyEvent.CTRL_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_LEFT))
        {
            ellipse.setFrame(ellipse.getX(), ellipse.getY(), ellipse.getWidth()-10, ellipse.getHeight());
            g.draw(ellipse);
        }

        if((e.getModifiers() == KeyEvent.CTRL_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_RIGHT))
        {
            ellipse.setFrame(ellipse.getX(), ellipse.getY(), ellipse.getWidth()+10, ellipse.getHeight());
            g.draw(ellipse);
        }

        /* The following are for translating the ellipse */
        if((e.getModifiers() == KeyEvent.SHIFT_MASK)&&
            (e.getKeyCode() == KeyEvent.VK_UP))
        {
            ellipse.setFrame(ellipse.getX(), ellipse.getY()+10, ellipse.getWidth(), ellipse.getHeight());
            g.draw(ellipse);
        }

        if((e.getModifiers() == KeyEvent.SHIFT_MASK)&&
            (e.getKeyCode() == KeyEvent.VK_DOWN))
        {
            ellipse.setFrame(ellipse.getX(), ellipse.getY()-10, ellipse.getWidth(), ellipse.getHeight());
            g.draw(ellipse);
        }

        if((e.getModifiers() == KeyEvent.SHIFT_MASK)&&
            (e.getKeyCode() == KeyEvent.VK_LEFT))
        {
            ellipse.setFrame(ellipse.getX()-10, ellipse.getY(), ellipse.getWidth(), ellipse.getHeight());
            g.draw(ellipse);
        }

        if((e.getModifiers() == KeyEvent.SHIFT_MASK)&&
            (e.getKeyCode() == KeyEvent.VK_RIGHT))
        {
            ellipse.setFrame(ellipse.getX()+10, ellipse.getY(), ellipse.getWidth(), ellipse.getHeight());
            g.draw(ellipse);
        }
    }

    //----------------------------------------------------------------------------
    public boolean mouseReleased(int x, int y, Graphics2D g)
    {
        g.setXORMode(Color.white);

        g.setColor(pen_color_);
        g.setStroke(stroke_);
        g.draw( ellipse );
        ellipse = newEllipse2D(x, y);
        g.setPaintMode();
        g.setColor(fill_color_);
        g.fill( ellipse );
        g.setColor(pen_color_);
        g.draw( ellipse );
        return ( (ellipse.getWidth() > 0.0) || (ellipse.getHeight() > 0.0) );
    }

    //----------------------------------------------------------------------------
    public String toSVG(long time)
    {
        double radius_x = ellipse.getWidth()  / 2.0;
        double radius_y = ellipse.getHeight() / 2.0;

        return "<" + DrawObjectFactory.SVG_ELLIPSE
        + " sender=\"" + getSender()
                + "\" id=\"" + getObjectId()
                + "\" cx=\"" + ( ellipse.getX() + radius_x )
                + "\" cy=\"" + ( ellipse.getY() + radius_y )
                + "\" rx=\"" + radius_x
                + "\" ry=\"" + radius_y + "\" "
                + toSVGFillColor()
                + toSVGStroke() + "/>";
    }

    //----------------------------------------------------------------------------
    protected void createAWTObject(Element xml_node, double scale)
    {
        double width  = 2.0 * Double.parseDouble( xml_node.getAttribute("rx") );
        double height = 2.0 * Double.parseDouble( xml_node.getAttribute("ry") );
        double x      = Double.parseDouble( xml_node.getAttribute("cx") ) - width/2.0;
        double y      = Double.parseDouble( xml_node.getAttribute("cy") ) - height/2.0;

        setAWTFillColor(xml_node);
        setAWTStroke(xml_node, scale);

        ellipse = new java.awt.geom.Ellipse2D.Double( x * scale, y * scale, width * scale, height * scale );
    }

    //----------------------------------------------------------------------------
    public void setPropertyX( String x )
    {
        ellipse = new java.awt.geom.Ellipse2D.Double( Double.parseDouble(x), ellipse.getY(), ellipse.getWidth(), ellipse.getHeight() );
        repaint();
    }

    //----------------------------------------------------------------------------
    public double getPropertyX()
    {
        return ellipse.getX();
    }

    //----------------------------------------------------------------------------
    public void setPropertyY( String y )
    {
        ellipse = new java.awt.geom.Ellipse2D.Double( ellipse.getX(), Double.parseDouble(y), ellipse.getWidth(), ellipse.getHeight() );
        repaint();
    }

    //----------------------------------------------------------------------------
    public double getPropertyY()
    {
        return ellipse.getY();
    }

    //----------------------------------------------------------------------------
    public void setPropertyWidth( String width )
    {
        ellipse = new java.awt.geom.Ellipse2D.Double( ellipse.getX(), ellipse.getY(), Double.parseDouble(width), ellipse.getHeight() );
        repaint();
    }

    //----------------------------------------------------------------------------
    public double getPropertyWidth()
    {
        return ellipse.getWidth();
    }

    //----------------------------------------------------------------------------
    public void setPropertyHeight( String height )
    {
        ellipse = new java.awt.geom.Ellipse2D.Double( ellipse.getX(), ellipse.getY(), ellipse.getWidth(), Double.parseDouble( height ) );
        repaint();
    }

    //----------------------------------------------------------------------------
    public double getPropertyHeight()
    {
        return ellipse.getHeight();
    }

    //----------------------------------------------------------------------------
    public void setPropertyFillColor( Color fill_color )
    {
        fill_color_ = fill_color;
        repaint();
    }

    //----------------------------------------------------------------------------
    public Color getPropertyFillColor()
    {
        return fill_color_;
    }

    //----------------------------------------------------------------------------
    public void setPropertyPenColor( Color pen_color )
    {
        pen_color_ = pen_color;
        repaint();
    }

    //----------------------------------------------------------------------------
    public Color getPropertyPenColor()
    {
        return pen_color_;
    }

    //----------------------------------------------------------------------------
    public void setPropertyStrokeWidth( String width )
    {
        stroke_ = new BasicStroke( Float.parseFloat( width ) );
        repaint();
    }

    //----------------------------------------------------------------------------
    public float getPropertyStrokeWidth()
    {
        return stroke_.getLineWidth();
    }

    //----------------------------------------------------------------------------
    public boolean contains(Point2D p)
    {
        return ellipse.intersects( p.getX() - 1.0, p.getY() - 1.0, 3.0, 3.0);
    }

    //----------------------------------------------------------------------------
    public void move(double x, double y)
    {
        ellipse = new Ellipse2D.Double( ellipse.getX() + x, ellipse.getY() + y, ellipse.getWidth(), ellipse.getHeight() );
    }
}
