/*
 *  Copyright (c) 2002
 *  bestsolution EDV Systemhaus GmbH,
 *  http://www.bestsolution.at
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Header: /usr/bestsolution/cvsroot/java/draw_swf/at/bestsolution/drawswf/drawobjects/Rectangle.java,v 1.35 2003/01/24 01:35:53 tom Exp $
 */

/*
 * Line.java
 *
 * Created on 21. Mai 2002, 22:57
 */

package at.bestsolution.drawswf.drawobjects;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.BasicStroke;

import javax.swing.JTable;
import javax.swing.table.TableModel;

import java.awt.event.KeyEvent;

import com.anotherbigidea.flash.movie.Movie;
import com.anotherbigidea.flash.movie.Frame;
import com.anotherbigidea.flash.movie.Shape;
import com.anotherbigidea.flash.movie.Instance;
import com.anotherbigidea.flash.structs.AlphaColor;

import org.w3c.dom.Element;

import at.bestsolution.drawswf.*;

/**
 *
 * @author  tom
 */
public class Rectangle extends DrawObject
{
    private Point2D start_point_;
    private Rectangle2D rect;
    private static int instance_counter = 1;
    private TableModel model_;
    private JTable options_table_;
    private int lastMousePointX;
    private int lastMousePointY;

    private MainWindow mWin;
    private int objectID;

    //----------------------------------------------------------------------------
    /** Creates a new instance of Line */
    public Rectangle(MainWindow mW)
    {
        super( "Rectangle ", mW);
        instance_counter++;
        setObjectId();
        mWin = mW;
        rect        = null;
        start_point_ = null;
    }

    public String getName() {
        return "Rectangle " + getObjectId();
    }

    public static void resetObjectCounter()
    {
        instance_counter = 1;
    }

    public int getObjectId() {
        return objectID;
    }

    public void setObjectId() {
        objectID = instance_counter - 1;
    }

    public void setObjectId(int ID) {
        objectID = ID;
    }

    //----------------------------------------------------------------------------
    public void drawObject(Movie movie, int layer, int speed)
    {
        Frame frame = movie.appendFrame();
        Shape shape = new Shape();
        AlphaColor line_color = new AlphaColor( pen_color_.getRed(), pen_color_.getGreen(), pen_color_.getBlue(), pen_color_.getAlpha() );
        AlphaColor fill_color = new AlphaColor( fill_color_.getRed(), fill_color_.getGreen(), fill_color_.getBlue(), fill_color_.getAlpha() );

        shape.defineFillStyle(fill_color);
        shape.defineLineStyle((stroke_.getLineWidth()), line_color);
        shape.setLineStyle(1);
        shape.setRightFillStyle(1);

        shape.drawAWTPathIterator(rect.getPathIterator(null));

        Instance instance = frame.placeSymbol(shape, 0, 0);

        // sleep in flash...
        for (int count = 0; count < 5; count++ )
        {
            movie.appendFrame();
        }
    }

    //----------------------------------------------------------------------------
    public void drawObject(Graphics2D g, DrawingPanel panel)
    {
        g.setStroke( stroke_ );
        g.setColor( fill_color_ );
        g.fill( rect );
        g.setColor(pen_color_);
        g.draw( rect );

        if (panel.isReplay() == true)
        {
            pause(100*ANIMATION_DELAY);
        }
    }

    //----------------------------------------------------------------------------
    private Rectangle2D.Double newRectangle2D(double end_x, double end_y)
    {
        double rx;
        double ry;
        double width;
        double height;
        double x = start_point_.getX();
        double y = start_point_.getY();

        if (x < end_x)
        {
            rx = x;
            width = end_x - x;
        }
        else
        {
            rx = end_x;
            width = x - end_x;
        }

        if (y < end_y)
        {
            ry = y;
            height = end_y - y;
        }
        else
        {
            ry = end_y;
            height = y - end_y;
        }

        return new Rectangle2D.Double(rx, ry, width, height);
    }

    //----------------------------------------------------------------------------
    public void mouseDragged(int x, int y, Graphics2D g) {
        setRectXPoint(x);
        setRectYPoint(y);
        g.setXORMode(Color.white);
        g.setColor(pen_color_);
        g.setStroke(stroke_);
        g.draw( rect );
        rect = newRectangle2D(x, y);
        g.draw( rect );
    }

    //----------------------------------------------------------------------------
    public void mousePressed(int x, int y, Graphics2D g) {
        setRectXPoint(x);
        setRectYPoint(y);
        g.setColor(pen_color_);
        g.setStroke(stroke_);
        start_point_ = new Point2D.Double(x, y);
        rect = newRectangle2D(x, y);
        g.draw( rect );
    }

    //----------------------------------------------------------------------------
    public boolean mouseReleased(int x, int y, Graphics2D g) {
        setRectXPoint(x);
        setRectYPoint(y);
        g.setXORMode(Color.white);
        g.setColor( pen_color_ );
        g.setStroke( stroke_ );
        g.draw( rect );
        rect = newRectangle2D( x, y );
        g.setPaintMode();
        g.setColor(fill_color_);
        g.fill( rect );
        g.setColor(pen_color_);
        g.draw( rect );
            return ( (rect.getWidth() > 0.0) || (rect.getHeight() > 0.0) );
    }

    public void setRectXPoint(int x)
    {
        lastMousePointX = x;
    }

    public int getRectXPoint()
    {
        return lastMousePointX;
    }

    public void setRectYPoint(int y)
    {
        lastMousePointY = y;
    }

    public int getRectYPoint()
    {
        return lastMousePointY;
    }

    public void keyPressed(Graphics2D g, KeyEvent e)
    {
        /* The following are for resizing a drawing object */
        if((e.getModifiers() == KeyEvent.CTRL_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_UP))
        {
            rect = newRectangle2D(getRectXPoint(),getRectYPoint()+10);
            setRectYPoint(getRectYPoint()+10);
            g.draw( rect );
        }

        if((e.getModifiers() == KeyEvent.CTRL_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_DOWN))
        {
            rect = newRectangle2D(getRectXPoint(),getRectYPoint()-10);
            setRectYPoint(getRectYPoint()-10);
            g.draw( rect );
        }

        if((e.getModifiers() == KeyEvent.CTRL_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_LEFT))
        {
            rect = newRectangle2D(getRectXPoint()-10,getRectYPoint());
            setRectXPoint(getRectXPoint()-10);
            g.draw( rect );
        }

        if((e.getModifiers() == KeyEvent.CTRL_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_RIGHT))
        {
            rect = newRectangle2D(getRectXPoint()+10,getRectYPoint());
            setRectXPoint(getRectXPoint()+10);
            g.draw( rect );
        }

        /* The following are for translating the rectangle */
        if((e.getModifiers() == KeyEvent.SHIFT_MASK) &&
            (e.getKeyCode() == KeyEvent.VK_UP))
        {
            rect.setRect(rect.getX(), rect.getY()-10, rect.getWidth(), rect.getHeight());
            g.draw(rect);
        }

        if((e.getModifiers() == KeyEvent.SHIFT_MASK)&&
            (e.getKeyCode() == KeyEvent.VK_DOWN))
        {
            rect.setRect(rect.getX(), rect.getY()+10, rect.getWidth(), rect.getHeight());
            g.draw(rect);
        }

        if((e.getModifiers() == KeyEvent.SHIFT_MASK)&&
            (e.getKeyCode() == KeyEvent.VK_LEFT))
        {
            rect.setRect(rect.getX()-10, rect.getY(), rect.getWidth(), rect.getHeight());
            g.draw(rect);
        }

        if((e.getModifiers() == KeyEvent.SHIFT_MASK)&&
            (e.getKeyCode() == KeyEvent.VK_RIGHT))
        {
            rect.setRect(rect.getX()+10, rect.getY(), rect.getWidth(), rect.getHeight());
            g.draw(rect);
        }
    }

    //----------------------------------------------------------------------------
    public String toSVG(long time)
    {
        return "<" + DrawObjectFactory.SVG_RECTANGLE
        + " id=\"" + getObjectId()
    + "\" sender=\"" + getSender()
        + "\" x=\"" + rect.getX()
        + "\" y=\"" + rect.getY()
        + "\" width=\"" + rect.getWidth()
        + "\" height=\"" + rect.getHeight() + "\" "
        + toSVGFillColor()
        + toSVGStroke() + "/>";
    }

    //----------------------------------------------------------------------------
    protected void createAWTObject(Element xml_node, double scale)
    {
        double width  = Double.parseDouble( xml_node.getAttribute("width") );
        double height = Double.parseDouble( xml_node.getAttribute("height") );
        double x      = Double.parseDouble( xml_node.getAttribute("x") );
        double y      = Double.parseDouble( xml_node.getAttribute("y") );

        setAWTFillColor(xml_node);
        setAWTStroke(xml_node, scale);

        rect = new Rectangle2D.Double( x * scale, y * scale, width * scale, height * scale );
    }

    //----------------------------------------------------------------------------
    public void setPropertyX( String x )
    {
        rect = new java.awt.geom.Rectangle2D.Double( Double.parseDouble(x), rect.getY(), rect.getWidth(), rect.getHeight() );
        repaint();
    }

    //----------------------------------------------------------------------------
    public double getPropertyX()
    {
        return rect.getX();
    }

    //----------------------------------------------------------------------------
    public void setPropertyY( String y )
    {
        rect = new java.awt.geom.Rectangle2D.Double( rect.getX(), Double.parseDouble(y), rect.getWidth(), rect.getHeight() );
        repaint();
    }

    //----------------------------------------------------------------------------
    public double getPropertyY()
    {
        return rect.getY();
    }

    //----------------------------------------------------------------------------
    public void setPropertyWidth( String width )
    {
        rect = new java.awt.geom.Rectangle2D.Double( rect.getX(), rect.getY(), Double.parseDouble(width), rect.getHeight() );
        repaint();
    }

    //----------------------------------------------------------------------------
    public double getPropertyWidth()
    {
        return rect.getWidth();
    }

    //----------------------------------------------------------------------------
    public void setPropertyHeight( String height )
    {
        rect = new java.awt.geom.Rectangle2D.Double( rect.getX(), rect.getY(), rect.getWidth(), Double.parseDouble( height ) );
        repaint();
    }

    //----------------------------------------------------------------------------
    public double getPropertyHeight()
    {
        return rect.getHeight();
    }

    //----------------------------------------------------------------------------
    public void setPropertyFillColor( Color fill_color )
    {
        fill_color_ = fill_color;
        repaint();
    }

    //----------------------------------------------------------------------------
    public Color getPropertyFillColor()
    {
        return fill_color_;
    }

    //----------------------------------------------------------------------------
    public void setPropertyPenColor( Color pen_color )
    {
        pen_color_ = pen_color;
        repaint();
    }

    //----------------------------------------------------------------------------
    public Color getPropertyPenColor()
    {
        return pen_color_;
    }

    //----------------------------------------------------------------------------
    public void setPropertyStrokeWidth( String width )
    {
        stroke_ = new BasicStroke( Float.parseFloat( width ) );
        repaint();
    }

    //----------------------------------------------------------------------------
    public float getPropertyStrokeWidth()
    {
        return stroke_.getLineWidth();
    }

    //----------------------------------------------------------------------------
    public boolean contains(Point2D p)
    {
        return rect.intersects( p.getX() - 1.0, p.getY() - 1.0, 3.0, 3.0);
    }

    //----------------------------------------------------------------------------
    public void move(double x, double y)
    {
        rect = new Rectangle2D.Double( rect.getX() + x, rect.getY() + y, rect.getWidth(), rect.getHeight() );
    }
}
