package ca.utoronto.atrc.whiteboard;

// Copyright 2003, Adaptive Technology Resource Centre
// Written by: Taras Kowaliw, taras.kowaliw@utoronto.ca

import java.io.*;
import java.util.Iterator;
import com.echomine.jabber.*;
//import org.w3c.dom.*;
import java.awt.*;
import javax.swing.*;
import java.util.Vector;
import java.util.Enumeration;

import org.jdom.Element;
import org.jdom.Attribute;
import org.jdom.Namespace;

public class ACommunicator {

    // echomine constructs
    public JabberContext context;
    public Jabber jabber;
    public JabberSession session;
    public JabberRosterService roster;
    public JabberChatService chat;
    public DefaultMessageParser parser;
    public JID theJID;

    // local user-defined variables
    public String userName;
    public String password;
    public String serverStr;
    public String myJID;
    public int port;
    public String presence = "offline";
    
    // gui items
    public RosterGUI rosterGUI;

    // utility constructs
    public RosterHash rosterHash;
    public ACommLoginOptions options;
    public ACommOptions jidOptions;
    public ACommOptionGUI optionGUI;
    public ACommToolKit toolKit = new ACommToolKit();
    public GroupChat theGroupChat;
    public String aCommGroupChatJID = "foo@foo.com";
    public String[] tempGroupChatList = new String[100];
    public String[] tempGroupChatBody = new String[100];
    public int tempGroupChatListCounter = 0;

    // user preference variables
    public boolean newestFirst = true;
    
    // Initialize everything, show login screen
    public ACommunicator() {
        System.out.println("AComm Test Application.\nThis is an Alpha Version.\n(c) 2003 Adaptive Technology Resource Centre, University of Toronto\n");
	
	String theFileStr = System.getProperty("user.home")+ File.separator + ".drawswf" + File.separator;
	File theFile = new File(theFileStr);
	if (!theFile.exists()) {
	    try { theFile.mkdir(); } catch (Exception e) { e.printStackTrace(); }
	}
	
	toolKit.setUI();
	
	//try { groupChatJID = new JID("foo@foo.com"); } catch (Exception e) { e.printStackTrace(); }
	
	options = new ACommLoginOptions();
	parser = new DefaultMessageParser();
	try {
	    parser.setParser("x",Namespace.getNamespace("atrc:utoronto:ca:whiteboard"),"ca.utoronto.atrc.whiteboard.WhiteboardXMessage");
	    //parser.setParser("query",Namespace.getNamespace("atrc:utoronto:ca:whiteboard"),"ca.utoronto.atrc.whiteboard.WhiteboardMessage");
	} catch (Exception e) {
	    //e.printStackTrace();
	}
        showLogin();
    }
    
    // Total shut down
    public void killItAll() {
        if (theGroupChat != null && theGroupChat.whiteboardWindow != null) theGroupChat.whiteboardWindow.killIt();
        if (rosterHash != null) {
	    Enumeration enum = rosterHash.theHash.keys();
            while (enum.hasMoreElements()) {
                String tJID = (String) enum.nextElement();
                Roster temp = rosterHash.getRoster(tJID);
	        if (temp.whiteboardWindow != null) temp.whiteboardWindow.killIt();
            }
	}
        if (session != null) session.disconnect();
        System.exit(0);
    }
    
    public void showOptionGUI() {
        if (optionGUI != null) {
	    optionGUI.theFrame.pack();
	    optionGUI.theFrame.setVisible(true);
	} else {
	    optionGUI = new ACommOptionGUI(this);
	}
    }

    // After login screen, configure with user options, connect
    public void doLogin(String password) {
        userName = options.p.getProperty("userName");
	serverStr = options.p.getProperty("server");
	myJID = userName + "@" + serverStr;
	rosterHash = new RosterHash(this);
	
	try {
	    theJID = new JID(myJID);
	} catch (Exception e) {
            JOptionPane.showMessageDialog(null, "<html><body><p align='center'>Error: Cannot process JID "+myJID+". Please try again.</p></body></html>");
	    showLogin();
	    return;	
	}
	
	try {
	    port = Integer.parseInt(options.p.getProperty("port"));
	} catch (Exception e) { 
	    JOptionPane.showMessageDialog(null, "<html><body><p align='center'>Error: Value for port ("+options.p.getProperty("port")+") not a number. Please try again.</p></body></html>");
	    showLogin();
	    return; 
	}
    
	JabberContext context = new JabberContext(userName, password, serverStr);
        Jabber jabber = new Jabber();
        session = jabber.createSession(context, parser);
	session.addMessageListener(new JabberMessageListener() {
	    public void messageReceived(JabberMessageEvent event) {
                processMessageEvent(event);
	    }
	});        

	try {	    
	    // connect...
	    session.connect(serverStr, port);
	    session.getUserService().login();
        } catch (Exception e) {
            //e.printStackTrace();
	    session.disconnect();
	    JOptionPane.showMessageDialog(null, "<html><body><p align='center'>There was an error connecting.<br>Please check your user name<br> and password and try again.</p></body></html>");
	    showLogin();
	    return;
        }	
	
	jidOptions = new ACommOptions(myJID);

	try {	    
	    // make roster GUI
	    rosterGUI = new RosterGUI(this);
        } catch (Exception e) {
            e.printStackTrace();
	    System.exit(0);
        }	
    }

    // Message listener for Jabber message events
    public void processMessageEvent(JabberMessageEvent event) {
        try {
	    //System.out.println("\n\n\nReceiving Message!:\n"+event.getMessage());
	    int type = event.getMessageType();

	    String tJID = null;
	    if (type == JabberCode.MSG_IQ_ROSTER) {
		eventRoster(event);
	    } else if (type == JabberCode.MSG_IQ_VERSION) {
		JabberIQMessage msg = (JabberIQMessage) event.getMessage();
		String iqType = msg.getType();
		if (msg.getFrom() != null) tJID = toolKit.jidWithoutEnd(msg.getFrom());
		//System.out.println("Got IQ_VERSION message: "+iqType+" from: "+tJID);
	        java.util.Date d = new java.util.Date();
		long time = d.getTime();
	    } else if (type == JabberCode.MSG_IQ) {
		JabberIQMessage msg = (JabberIQMessage) event.getMessage();
		String iqType = msg.getType();
		if (msg.getFrom() != null) tJID = toolKit.jidWithoutEnd(msg.getFrom());
		//System.out.println("Got IQ message: "+iqType+" from: "+tJID);
		java.util.Date d = new java.util.Date();
		long time = d.getTime();
	    } else if (type == JabberCode.MSG_CHAT) {
		JabberChatMessage msgT = (JabberChatMessage) event.getMessage();
		JabberMessage xMsg = null;
		xMsg = msgT.getXMessage("atrc:utoronto:ca:whiteboard");
		//if (xMsg == null) System.out.println("NO X MESSAGE!");
		eventChat(msgT, xMsg);
	    } else if (type == JabberCode.MSG_PRESENCE) { 
		JabberPresenceMessage msg = (JabberPresenceMessage) event.getMessage();
		String roomJID = toolKit.jidWithoutEnd(msg.getFrom());
		if (theGroupChat!=null && roomJID.equalsIgnoreCase(aCommGroupChatJID)) groupChatPresence(msg);
		else if (roomJID.equalsIgnoreCase(aCommGroupChatJID)) earlyGroupChatPresence(msg);
		String pType = msg.getType();
		if (pType.equalsIgnoreCase("error")) eventPresenceError(msg);
		else if (pType.equalsIgnoreCase("unsubscribed")) eventPresenceUnsubscribed(msg);
		else if (pType.equalsIgnoreCase("subscribed")) eventPresenceSubscribed(msg);
		else if (pType.equalsIgnoreCase("subscribe")) eventPresenceSubscribe(msg);
		else eventPresence(msg);
	    } 
	} catch (Exception e) { e.printStackTrace(); 
	}
    }

    public void showLogin() {
        ACommLogin aLogin = new ACommLogin(this);
    }   
    
    // ****************************************************************
    // ****************************************************************
    // ****************************************************************
    // ***************** Initializing Group Chat GUI ******************
    public void startGroupChat(String nickName, String roomJID) {
        aCommGroupChatJID = roomJID;

	//System.out.println("JOINING GROUP CHAT NOW!!");
        if (theGroupChat == null || !theGroupChat.theFrame.isVisible()) {
	    theGroupChat = null;
            try {
	        chat = session.getChatService();
	        chat.joinChatRoom(roomJID, nickName, false);
	    } catch (Exception e) {
	        e.printStackTrace();
	        return;
	    }	    
	    
	} else {
	    JOptionPane.showMessageDialog(null, "<html><body><p align='center'>Error: You may be conected to<br>at most one Group Chat room at a time.</p></body></html>");
	}
	//System.out.println("MAKING GROUP CHAT GUI NOW!!");
	theGroupChat = new GroupChat(nickName, roomJID, this);
    } 
    
    public void leaveGroupChat(String roomJID) {
        try {
	    chat.leaveChatRoom(roomJID, false);
	} catch (Exception e) {
	    //e.printStackTrace();
	    return;
	}  
    }
    
    // ****************************************************************
    // ****************************************************************
    // ***************** Managing User's Roster List ******************
    public void removeContact(String removeJID) {
    	try {
            rosterHash.remove(removeJID);
	    roster.removeFromRoster(removeJID, false);
	} catch (Exception e) { 
	    //e.printStackTrace(); 
	}
    }
    
    public void subscribeContact(String tJID) {
        try {
	    session.getPresenceService().subscribe(tJID);
	} catch (Exception e) { 
	    //e.printStackTrace(); 
	}    
    }
    
    public void addContact(String addJID) {
        try {
	    roster.addToRoster(addJID, "", "", false);
	    rosterHash.addRosterItem(addJID, "unsubscribed");
	} catch (Exception e) { 
	    //e.printStackTrace(); 
	}
    }
    
    // *****************************************************************
    // *****************************************************************
    // ***************** Handling Jabber Message Events ****************
    
    public void groupChatPresence(JabberPresenceMessage msg) {
	String tJID = toolKit.jidWithoutEnd(msg.getFrom());
	String nickName = toolKit.jidResource(msg.getFrom());
	String pType = msg.getType();  
	theGroupChat.setPresence(nickName, pType);
	theGroupChat.printUserList();
    }

    public void earlyGroupChatPresence(JabberPresenceMessage msg) {
        //System.out.println("@@@@@@@@@@@@@@@@@@@@@Early group cat presence");
	String tJID = toolKit.jidWithoutEnd(msg.getFrom());
	String nickName = toolKit.jidResource(msg.getFrom());
	String pType = msg.getType();  
	String groupChatNickName = toolKit.jidResource(msg.getFrom());
	if (groupChatNickName == null) groupChatNickName = "system";
	tempGroupChatList[tempGroupChatListCounter] = groupChatNickName;
	tempGroupChatBody[tempGroupChatListCounter] = null;
	tempGroupChatListCounter ++;	
    }    
        
    public void eventRoster(JabberMessageEvent event) {
        RosterIQMessage msg = (RosterIQMessage) event.getMessage();
	Iterator items = msg.getRosterItems().iterator();
	RosterItem item;
	while (items.hasNext()) {
            item = (RosterItem) items.next();
	    if (item.getSubscription().equalsIgnoreCase("remove")) {
		
	    } else {
		rosterHash.addRosterItem(item);
            }
	}
    }
    
    public void eventChat(JabberChatMessage msg, JabberMessage xMsg) {
        String tJID = toolKit.jidWithoutEnd(msg.getFrom());
	String groupChatNickName = toolKit.jidResource(msg.getFrom());
        java.util.Date d = new java.util.Date();
	long time = d.getTime();	
	Message m = new Message(tJID, msg.getSubject(), msg.getBody(), "", time);
	
	WhiteboardXMessage wXMsg = null;
	Message descMsg = null;	
	if (xMsg != null) {
	    wXMsg = (WhiteboardXMessage) xMsg;
	    //if (wXMsg.messageStr != null) descMsg = new Message("system", "", wXMsg.messageStr, "", time);
	    //System.out.println("@@@@@@@@@@ Got an X Message!!! @@@@@@@@@@@");
	    //if (wXMsg == null) System.out.println("not a whiteboard message?");
	    if (msg.getType().equals(JabberChatMessage.TYPE_ERROR)) {
	    
	    } else if (xMsg.getMessageType() == 6543 && msg.getType().equals(JabberChatMessage.TYPE_GROUPCHAT)) {
	        //System.out.println("Group Chat message!, from: "+msg.getFrom()+", resource: "+groupChatNickName);
		if (theGroupChat != null) {
		    theGroupChat.newWhiteboardMessage(wXMsg, groupChatNickName);
		    if (descMsg != null) theGroupChat.newGroupChatMessage(descMsg);
		    if (jidOptions.getBoolean("playChatSound")) toolKit.playNewChatSound();
		}
	    } else if (xMsg.getMessageType() == 6543 ) {
	        //if (wXMsg == null) System.out.println("wXMsg - It's null!");
	    
		Roster temp = rosterHash.getRoster(tJID);
	        temp.newWhiteboardMessage(wXMsg, msg.getFrom());
		if (descMsg != null) temp.newChatMessage(descMsg);
		if (jidOptions.getBoolean("playChatSound")) toolKit.playNewChatSound();
	    } 
	}
	
			
	if (msg.getType().equals(JabberChatMessage.TYPE_ERROR)) {
	    JOptionPane.showMessageDialog(null, "<html><body><p align='center'>Error in sending message to <i>"+tJID+"</i>:<br>"+msg.getErrorMessage()+"</p></body></html>");
	    return;
	} else if (msg.getType().equals(JabberChatMessage.TYPE_GROUPCHAT)) {
	    //System.out.println("Got a new group chat message! From: "+msg.getFrom()+", message: "+msg.getBody());
	    if (theGroupChat == null) {
	        tempGroupChatList[tempGroupChatListCounter] = groupChatNickName;
		tempGroupChatBody[tempGroupChatListCounter] = msg.getBody();
	        tempGroupChatListCounter ++;
	        //System.out.println("ERROR: message, but no groupchat defined!");
	    } else {
	        if (groupChatNickName != null) m.sJID = groupChatNickName;
		else m.sJID = "system";
	        theGroupChat.newGroupChatMessage(m);
	    }
	    if (jidOptions.getBoolean("playChatSound")) toolKit.playNewChatSound();
	    
	} else if (msg.getType().equals(JabberChatMessage.TYPE_CHAT)) {
	    //System.out.println("Got a private Chat message... " + msg.getBody());
	    Roster temp = rosterHash.getRoster(tJID);
	    temp.newChatMessage(m);
	    if (jidOptions.getBoolean("playChatSound")) toolKit.playNewChatSound();
	    
	} else if (msg.getType().equals(JabberChatMessage.TYPE_NORMAL)) {
	    Roster temp = rosterHash.getRoster(tJID);
	    temp.newNormalMessage(m);
	    if (jidOptions.getBoolean("playMessageSound")) toolKit.playNewMessageSound(); 
	}
    }

    public void eventPresence(JabberPresenceMessage msg) {
	String tJID = toolKit.jidWithoutEnd(msg.getFrom());
	String tStatus = msg.getStatus();
	String tShowState = msg.getShowState();
	String pType = msg.getType();  
	rosterHash.setPresence(tJID, pType, tStatus, tShowState);
    }     
    
    public void eventPresenceSubscribe(JabberPresenceMessage msg) {
	String tJID = toolKit.jidWithoutEnd(msg.getFrom());
	String tStatus = msg.getStatus();
	String tShowState = msg.getShowState();
	String pType = msg.getType();  
	int n = JOptionPane.showConfirmDialog(null, "<html><body><p><i>"+tJID+"</i> is requesting a subscription<br> to your Presence information. This means that he/she<br> will be informed when you are online/offline, and vica versa.<br> <b>Allow?</b></p></body></html>", "Request for Subscription", JOptionPane.YES_NO_OPTION);
	try {
	    if (n == JOptionPane.YES_OPTION) {
	        session.getPresenceService().acceptSubscribe(msg);
                //System.out.println("More stuff here...");
	    } else {
	        session.getPresenceService().denySubscribe(msg);
	    }
	} catch (Exception e) { 
	    //e.printStackTrace(); 
	}
    }        
    
    public void eventPresenceSubscribed(JabberPresenceMessage msg) {
	String tJID = toolKit.jidWithoutEnd(msg.getFrom());
	String tStatus = msg.getStatus();
	String tShowState = msg.getShowState();
	String pType = msg.getType();  
	Roster temp = rosterHash.getRoster(tJID);
        if (temp != null) {
            temp.subscription = "subscribed";
	    temp.updateRosterButtons();
	} //else System.out.println("ERROR: ***** eventPresenceSubscribed - no such JID *****");  
    }    
    
    public void eventPresenceUnsubscribed(JabberPresenceMessage msg) {
	String tJID = toolKit.jidWithoutEnd(msg.getFrom());
	String tStatus = msg.getStatus();
	String tShowState = msg.getShowState();
	String pType = msg.getType();  
	Roster temp = rosterHash.getRoster(tJID);
	if (temp != null) {
	    if (!temp.isSubscribed()) {
		JOptionPane.showMessageDialog(null, "<html><body><p align='center'>Contact <i>"+tJID+"</i> has not authorized a subscription to his/her Presence information.</p></body></html>");
	    } else {
		JOptionPane.showMessageDialog(null, "<html><body><p align='center'>Contact <i>"+tJID+"</i> has cancelled your subscription to his/her Presence information.</p></body></html>");		    
	    }
	    temp.subscription = "unsubscribed";
	    temp.updateRosterButtons();
	} //else System.out.println("ERROR: ***** eventPresenceUnsubscribed - No such roster in hash *****");       
    }
            
    public void eventPresenceError(JabberPresenceMessage msg) {
	String tJID = toolKit.jidWithoutEnd(msg.getFrom());
	String tStatus = msg.getStatus();
	String tShowState = msg.getShowState();
	String pType = msg.getType();  
	Roster temp = rosterHash.getRoster(tJID);
	if (tJID.equalsIgnoreCase(aCommGroupChatJID)) {
		JOptionPane.showMessageDialog(null, "<html><body><p align='center'>ERROR: There was an error connecting to the group chat:<br><i>"+msg.getErrorMessage()+"</i><br>(Try another Nick Name)</p></body></html>");	
		if (theGroupChat != null) {
		    theGroupChat.theFrame.setVisible(false);
		    theGroupChat.theFrame.dispose();
		    theGroupChat = null;
		    
		}	
		ACommGroupChatLogin gCLogin = new ACommGroupChatLogin(this);    	   
	}
	if (temp != null) {
	    String errorStr = "<html><body><p align='center'>Error in adding contact <i>"+tJID+"</i>:<br>"+msg.getErrorMessage()+"<br>Do you want this Contact removed from your Roster?</p></body></html>";
	    int n = JOptionPane.showConfirmDialog(null, errorStr, "Error in Subscribing", JOptionPane.YES_NO_OPTION);
	    if (n == JOptionPane.YES_OPTION) {
		removeContact(tJID);
	    }
	} //else System.out.println("ERROR: ***** eventPresenceError - No such roster in hash *****");      
    }
    
    // *****************************************************************
    // *****************************************************************
    // ***************** Sending Chat Messages *************************
    
    // Request ... for GROUPCHAT ONLY!!
    public void sendWhiteboardRequest(String jid) {
	Element x = new Element("x", Namespace.getNamespace("atrc:utoronto:ca:whiteboard"));
	Element wbNode = new Element("whiteboard");
	wbNode.setAttribute("type", "req");
	x.addContent(wbNode);    
	WhiteboardXMessage msgX = new WhiteboardXMessage(x);
	JabberChatMessage msg = new JabberChatMessage("groupchat");
	msg.setTo(jid);
	msg.setBody(null);
	msg.setXMessage("atrc:utoronto:ca:whiteboard", msgX);
	try {
	    session.sendMessage(msg);
	} catch (Exception e) {
	    e.printStackTrace();
	}	    
    }
    

    public void sendWhiteboardMessage(String jid, String type, Vector svgVector, Vector svgRMVector, Vector descVector) {   
        sendWhiteboardMessage(jid, type, svgVector, svgRMVector, descVector, null, null);
    } 

    public void sendWhiteboardMessage(String jid, String type, Vector svgVector, Vector svgRMVector, Vector descVector, String theDesc) {   
        sendWhiteboardMessage(jid, type, svgVector, svgRMVector, descVector, theDesc, null);
    } 
        
    public void sendWhiteboardMessage(String jid, String type, Vector svgVector, Vector svgRMVector, Vector descVector, String theDesc, String toWhom) {
	//System.out.println("Sending whiteboard message. "+jid+" X "+type+" X "+ theDesc);
	Roster temp = rosterHash.getRoster(jid);
	if (temp != null && theDesc != null && theDesc != "") {
	    temp.whiteboardChatMessage(theDesc);
	}
	
	Element xM = new Element("x", Namespace.getNamespace("atrc:utoronto:ca:whiteboard"));
	Element wbNode = new Element("whiteboard");
	wbNode.setAttribute("type", type);	
	if (toWhom != null) wbNode.setAttribute("who", toWhom);	
	else 	wbNode.setAttribute("who", "");	
	xM.addContent(wbNode);
	Element svgNode = new Element("svg");
	if (svgVector != null) {
	    for (int i=0; i<svgVector.size(); i++) {
	    
	        //System.out.println((String) svgVector.elementAt(i));
	        Element svgChild = toolKit.stringToElement((String) svgVector.elementAt(i));
		if (svgChild != null) svgNode.addContent(svgChild);
		//else System.out.println("It's null!");
	    }
	}
	Element svgRMNode = new Element("svg_rm");
	if (svgRMVector != null) {
	    for (int i=0; i<svgRMVector.size(); i++) {
	        Element svgRMChild = toolKit.stringToElement((String) svgRMVector.elementAt(i));
		svgRMNode.addContent(svgRMChild);
	    }
	}
	Element dLNode = new Element("descList");
	if (descVector != null) {
	    for (int i=0; i<descVector.size(); i++) {
	        Element descChild = toolKit.stringToElement((String) descVector.elementAt(i));
		dLNode.addContent(descChild);
	    }
	}
	Element msgNode = new Element("chat_message");
	msgNode.setText(theDesc);
	
	wbNode.addContent(svgNode);
	wbNode.addContent(svgRMNode);
	wbNode.addContent(dLNode);	
	wbNode.addContent(msgNode);
	
	WhiteboardXMessage msgX = new WhiteboardXMessage(xM);
	JabberChatMessage msg = null;
	if (theGroupChat != null && theGroupChat.roomJID.equalsIgnoreCase(jid)) msg = new JabberChatMessage("groupchat");
	else msg = new JabberChatMessage("chat");	
	msg.setTo(jid);
	msg.setBody("");
	msg.setXMessage("atrc:utoronto:ca:whiteboard", msgX);
	//System.out.println("Added X message");
	
	//System.out.println("\n\n\nSending Message!:\n"+msg+"");
	
	try {
	    session.sendMessage(msg);
	} catch (Exception e) {
	    e.printStackTrace();
	}	
    }    
        
    public void sendGroupChatMessage(String roomName, String message) {
        //System.out.println("\nSending group chat message! roomName: "+roomName+", message: "+message);
	try {
	    chat.sendChatMessage(roomName, message, false);
	} catch (Exception e) {
	    //e.printStackTrace();
	}    
    }

    public void sendPrivateChatMessage(String jid, String message) {
	sendMessage(jid, "chat", "", message, "");
	java.util.Date d = new java.util.Date();
	long time = d.getTime();
	Message m = new Message(userName, "", message, "", time);
	Roster temp = rosterHash.getRoster(jid);
	temp.newChatMessage(m);
    }
        
    public void sendNormalMessage(String jid, String subject, String message) {
	sendMessage(jid, "normal", subject, message, "");
	java.util.Date d = new java.util.Date();
	long time = d.getTime();
	Message m = new Message(userName, subject, message, "", time);
	Roster temp = rosterHash.getRoster(jid);
	temp.newNormalMessage(m);
    }

    public void sendMessage(String jid, String type, String subject, String message, String threadID) {
	JabberChatMessage msg = new JabberChatMessage(type);
	msg.setTo(jid);
	msg.setBody(message);
	if (subject != null && subject != "") msg.setSubject(subject);
	if (threadID != null && threadID != "") msg.setThreadID(threadID);
	try {
	    session.sendMessage(msg);
	} catch (Exception e) {
	    //e.printStackTrace();
	}
	//System.out.println("Sending message of type "+type+" to "+jid);
    }    
    
    // *****************************************************************
    // *****************************************************************
    // ***************** Manage User's Presence ************************
    public void setOnline() {
        try {
	    if (presence.equalsIgnoreCase("offline")) {
	        roster = session.getRosterService();
	        roster.requestRosterList(false);
	    }
            session.getPresenceService().setToAvailable(null,null,false);
	} catch (Exception e) {
	    //e.printStackTrace();
	}
        presence = "online";
    }
    
    public void setNA() {
        try {
	    if (presence.equalsIgnoreCase("offline")) {
	        roster = session.getRosterService();
	        roster.requestRosterList(false);
	    }
	    session.getPresenceService().setToAvailable("xa",options.p.getProperty("xa"), false);
	} catch (Exception e) {
	    //e.printStackTrace();
	}
        presence = "na";
    }
    
    public void setDND() {
        try {
	    if (presence.equalsIgnoreCase("offline")) {
	        roster = session.getRosterService();
	        roster.requestRosterList(false);
	    }
	    session.getPresenceService().setToAvailable("dnd",options.p.getProperty("dnd"), false);
	} catch (Exception e) {
	    //e.printStackTrace();
	}
        presence = "dnd";
    }
    
    public void setAway() {
        try {
	    if (presence.equalsIgnoreCase("offline")) {
	        roster = session.getRosterService();
	        roster.requestRosterList(false);
	    }
	    session.getPresenceService().setToAvailable("away",options.p.getProperty("away"), false);
	} catch (Exception e) {
	    //e.printStackTrace();
	}
        presence = "away";
    }    
    
    public void setOffline() {
        try {
            session.getPresenceService().setToUnavailable();
	} catch (Exception e) {
	    //e.printStackTrace();
	}
        presence = "offline";
	rosterHash.setAllUnsubscribed();
    }
}
