package ca.utoronto.atrc.whiteboard;

// Copyright 2003, Adaptive Technology Resource Centre
// Written by: Taras Kowaliw, taras.kowaliw@utoronto.ca

import java.io.*;
import com.echomine.jabber.*;
import java.util.Vector;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import at.bestsolution.drawswf.MainWindow;

public class GroupChat {

    public FileOutputStream theStream;
    public PrintWriter theWriter;

    public Vector chatMessages;
    
    // Group Chat GUI items
    public ACommGroupChatGUI groupChatPanel;
    public ACommGroupChatList groupChatList;  
        
    public JFrame theFrame;  
    public JFrame listFrame;
    
    //public ACommWhiteboard theWhiteboard;
    public MainWindow whiteboardWindow;
    
    // The original class
    public ACommunicator aComm;
    
    public String nickName;
    public String roomJID;
    
    // the list for group chat presence
    public Vector userList;
    
    public Vector whiteboardMessages;
    
    public boolean isWhiteboardVisible = false;
    
    public GroupChat(String nN, String rm, ACommunicator aC) {
        aComm = aC;
	nickName = nN;
	roomJID = rm;
	theFrame = null;    
        chatMessages = new Vector();
	whiteboardMessages = new Vector();
	userList = new Vector();
	groupChatPanel = new ACommGroupChatGUI(aComm, this);
	groupChatList = new ACommGroupChatList(aComm, this);
	//theWhiteboard = new ACommWhiteboard(roomJID, aComm);
	whiteboardWindow = new MainWindow(aComm, roomJID, aComm.rosterGUI.theFrame, true);
	whiteboardWindow.drawIt();
	//System.out.println("Making the Main Window");
	theFrame = new JFrame("AComm: Group Chat at "+roomJID);
	theFrame.addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent e) {
	       listFrame.setVisible(false);
	       listFrame.dispose();
	       theFrame.setVisible(false);
	       theFrame.dispose();
	       aComm.leaveGroupChat(roomJID);
           }});
	listFrame = new JFrame("AComm: Participants List");
	listFrame.addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent e) {
	       listFrame.setVisible(false);
	       listFrame.dispose();
           }});
	
	theFrame.getContentPane().setLayout(new BoxLayout(theFrame.getContentPane(), BoxLayout.X_AXIS));
	listFrame.getContentPane().add(groupChatList);	
	theFrame.getContentPane().add(groupChatPanel);
	//theFrame.getContentPane().add(whiteboardWindow);
	isWhiteboardVisible = false;
	theFrame.pack();
	listFrame.pack();	
	theFrame.setVisible(true);
	
	addMeToPresence();
	
	for (int i=0; i<aComm.tempGroupChatListCounter; i++) {
	    //System.out.println("Constructor! Adding: "+i+" "+aComm.tempGroupChatList[i]);
	    if (aComm.tempGroupChatList[i] != null && aComm.tempGroupChatList[i].equals(nickName)) addToPresence(aComm.tempGroupChatList[i]);
	    aComm.tempGroupChatList[i] = "";
	}
	aComm.tempGroupChatListCounter = 0;
	
	aComm.sendWhiteboardRequest(roomJID);
	
	try {
	    String path = System.getProperty("user.home")+ File.separator + ".drawswf"+File.separator+"transcript.html";
	    theStream = new FileOutputStream(path, true);
	    theWriter = new PrintWriter(theStream);
	} catch (Exception e) { e.printStackTrace(); }
	
	
    }
    
    public void showHideWhiteboard() {
        if (isWhiteboardVisible) removeWhiteboard();
        else addWhiteboard();
    }
    
    public void addWhiteboard() {
        theFrame.getContentPane().add(whiteboardWindow);
	theFrame.pack();
	isWhiteboardVisible = true;
    }
    
    public void removeWhiteboard() {
        theFrame.getContentPane().remove(whiteboardWindow);
	theFrame.pack();
	isWhiteboardVisible = false;
    }
    
    public void showList() {
        if (!listFrame.isVisible()) {
	    listFrame.setVisible(true);
	}
    }
    
    public void addToListModel(Message m) {
        String colour = "666666";
	if (m.sJID == null || m.sJID.equalsIgnoreCase("system")) colour = "44aa33";
	else if (!m.sJID.equals(nickName)) colour="000000";
        groupChatPanel.listModel.insertElementAt(m.sJID+">>> "+m.message+"@@@"+colour, 0);
    }
    
    public void whiteboardGroupChatMessage(String theMessage) {
        java.util.Date d = new java.util.Date();
	long time = d.getTime();
	Message m = new Message("system", "", theMessage, "", time); 
	newGroupChatMessage(m);   
    }
        
    public void newGroupChatMessage(Message m) {
        if (m.message != null && m.message != "") {
	    if (m.sJID != null && !m.sJID.equalsIgnoreCase("system")) addToPresence(m.sJID);
	    chatMessages.add(m);
	    addToListModel(m);
	    try {
		java.util.Date theDate = new java.util.Date();
		theWriter.println("<tr><td>"+theDate.toString()+"</td><td>"+m.sJID+"</td><td>"+m.message+"</td></tr>\n");
		theWriter.flush();
	    } catch (Exception e) { e.printStackTrace(); }	  
	}
    }
    
    public void newWhiteboardMessage(WhiteboardXMessage msg, String sender) {
	if (sender != null && !sender.equalsIgnoreCase("system")) addToPresence(sender);
	WhiteboardMessage tMsg = new WhiteboardMessage(sender, msg.getType(), msg.getSVG(), msg.getSVGRM(), msg.getDesc(), msg.getMessage(), msg.getToWhom());
	whiteboardGroupChatMessage(msg.getMessage());
	if (!sender.equalsIgnoreCase(nickName)) {
            whiteboardWindow.incomingWhiteboardMessage(sender, msg.getType(), msg.getSVG(), msg.getSVGRM(), msg.getDesc(), msg.getToWhom());
	} else {
            whiteboardWindow.incomingWhiteboardMessageFromMyself(sender, msg.getType());	
	}
	whiteboardMessages.add(tMsg);    
	
	try {
	    String theMessage = msg.getMessage();
	    if (theMessage == null || theMessage.length() < 1) theMessage = "<i>Whiteboard event: no message</i>";
	    java.util.Date theDate = new java.util.Date();
	    theWriter.println("<tr><td>"+theDate.toString()+"</td><td>"+sender+"</td><td>"+theMessage+"</td></tr>\n");
	    theWriter.flush();
	    System.out.println("<tr><td>"+theDate.toString()+"</td><td>"+sender+"</td><td>"+theMessage+"</td></tr>\n");
	} catch (Exception e) { e.printStackTrace(); }	  
    }
    
    public void setPresence(String tNickName, String pType) {
        //System.out.println("**** Set Presence! Nick Name: "+tNickName+", pType: "+pType);
	
	if (pType == null || pType.equalsIgnoreCase("available")) {
	    addToPresence(tNickName);
	} else {
	    removeFromPresence(tNickName);
	}
    }
    
    public void addMeToPresence() {
        String tNickName = "<html><body><font color='666666'>"+nickName+ " (you)</font></body></html>";
	groupChatList.listModel.addElement(tNickName);
    }
    
    public void addToPresence(String tNickName) {
        boolean equalsNN = (tNickName.equals(nickName));
	//System.out.println("X"+tNickName+"X = X"+nickName+"? : "+equalsNN);
        if (tNickName.equals(nickName)) return;
        for (int i=0; i<groupChatList.listModel.size(); i++) {
	    String temp = (String) groupChatList.listModel.elementAt(i);
	    if (temp.equals(tNickName)) {
		return;
	    }
	}
	groupChatList.listModel.addElement(tNickName);
    }
    
    public void removeFromPresence(String tNickName) {
        for (int i=0; i<groupChatList.listModel.size(); i++) {
	    String temp = (String) groupChatList.listModel.elementAt(i);
	    if (temp.equals(tNickName)) {
	        groupChatList.listModel.removeElementAt(i);
		return;
	    }
	}
    }
    
    public void printUserList() {
        //System.out.print("User List: ");
        for (int i=0; i<userList.size(); i++) {
	    String temp = (String) userList.elementAt(i);
	    System.out.print(" "+temp+" ");
	}    
	System.out.println("");
    }
}






