package ca.utoronto.atrc.whiteboard;

// Copyright 2003, Adaptive Technology Resource Centre
// Written by: Taras Kowaliw, taras.kowaliw@utoronto.ca

import com.echomine.jabber.*;
import java.util.Vector;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import at.bestsolution.drawswf.MainWindow;

public class Roster {

    public String jid;
    public String name;
    public RosterItem rosterItem;
    public String pType;
    public String pStatus;
    public String pShowState;
    public String subscription = "unknown";
    public Vector chatMessages;
    public Vector normalMessages;
    public Vector whiteboardMessages;
    public boolean unseenChat = false;
    public boolean unseenNormal = false;
    
    public boolean isWhiteboardVisible = false;
    
    // Private Chat GUI items
    public ACommPrivateChatGUI privateChatPanel;
    public JFrame theFrame;  
    //public ACommWhiteboard theWhiteboard;
    public MainWindow whiteboardWindow;

    // GUI elements for RosterGUI
    public JLabel jidLabel;
    public JButton chatButton;
    public JButton normalButton;
    
    // The original class
    public ACommunicator aComm;
    
    public Roster(String rJID, String pres, ACommunicator aC) {
        aComm = aC;
	jid = rJID;
	pType = "offline";
	getSubscription();
	makeIt();
    }
       
    public Roster(String rJID, ACommunicator aC, String sub) {
        aComm = aC;
	jid = rJID;
	pType = "offline";
	//getSubscription();
	subscription = sub;
	makeIt();
    }
    
    public Roster(RosterItem rI, ACommunicator aC) {
	aComm = aC;
	rosterItem = rI;
	jid = rI.getJID();
	pType = "offline";
	getSubscription();
	if (!isSubscribed()) {
	    String tSub = rI.getSubscription();
	    if (tSub != null && tSub.length() > 0) {
	        if (tSub.equalsIgnoreCase("both")) makeSubscribed();
	        else makeUnsubscribed();
	    }
	}
	makeIt();
    }
        
    public void makeIt() {
	theFrame = null;    
        chatMessages = new Vector();
	normalMessages = new Vector();
	whiteboardMessages = new Vector();
	
	// Initialize stuff for RosterGUI
	jidLabel = new JLabel(shortName(jid) + " (" + availability(shortName(jid)) + ")");
	jidLabel.setForeground(jidColour(shortName(jid)));
	chatButton = new JButton(getChatIcon());
	chatButton.setToolTipText(getChatText());
	chatButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		        if (aComm.presence.equalsIgnoreCase("offline")) 
			    JOptionPane.showMessageDialog(aComm.rosterGUI.rosterPanel, "You must be connected to the server to send messages. Select 'Online' to connect.");
			else showPCGUI(); 
		}
	    });	
	normalButton = new JButton(getNormalIcon());
	normalButton.setToolTipText(getNormalText());
	normalButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		        if (aComm.presence.equalsIgnoreCase("offline")) 
			    JOptionPane.showMessageDialog(aComm.rosterGUI.rosterPanel, "You must be connected to the server to send messages. Select 'Online' to connect.");
			else showNCGUI(); 
		}
	    });	
	
	privateChatPanel = new ACommPrivateChatGUI(aComm, this);
    }
    
    public void showNCGUI() {
        unseenNormal = false;
	//System.out.println("Normal Chat GUI! Coming soon to an AComm near you!");
	JOptionPane.showMessageDialog(aComm.rosterGUI.rosterPanel, "Normal Messages not yet implemented.");
	updateRosterButtons();
    }
    
    public void showHideWhiteboard() {
        if (isWhiteboardVisible) removeWhiteboard();
        else addWhiteboard();
    }
    
    public void addWhiteboard() {
        theFrame.getContentPane().add(whiteboardWindow);
	theFrame.pack();
	isWhiteboardVisible = true;
    }
    
    public void removeWhiteboard() {
        theFrame.getContentPane().remove(whiteboardWindow);
	theFrame.pack();
	isWhiteboardVisible = false;
    }

    public void showPCGUI() {
    	unseenChat = false;
	if (theFrame == null) {
	    theFrame = new JFrame("AComm: Private Chat with "+shortName(jid));
	    whiteboardWindow = new MainWindow(aComm, jid, aComm.rosterGUI.theFrame, false);
	    whiteboardWindow.drawIt();    
	    theFrame.getContentPane().setLayout(new BoxLayout(theFrame.getContentPane(), BoxLayout.X_AXIS));
	    theFrame.getContentPane().add(privateChatPanel);	    
	    //theFrame.getContentPane().add(whiteboardWindow);

	}
	theFrame.pack();
	theFrame.setVisible(true);
	
	for (int i=0; i<whiteboardMessages.size(); i++) {
            WhiteboardMessage tMsg = (WhiteboardMessage) whiteboardMessages.elementAt(i);
	    if (!tMsg.isSeen) whiteboardWindow.incomingWhiteboardMessage(tMsg.sender, tMsg.type, tMsg.svgVector, tMsg.svgRMVector, tMsg.descVector, tMsg.toWhom);	    
	    tMsg.isSeen = true;
	}
	// update the rosterHash GUI so that the incoming event
	// notice associated with this message is removed
	updateRosterButtons();
    }
    
    public void addToListModel(Message m) {
        String colour = "666666";
	if (m.sJID == null || m.sJID.equalsIgnoreCase("system")) colour = "44aa33";
	else if (m.sJID.equalsIgnoreCase(jid)) colour="000000";
        privateChatPanel.listModel.insertElementAt(shortName(m.sJID)+">>> "+m.message+"@@@"+colour, 0);
        //listModel.addElement(shortName(m.sJID)+"> "+m.message);
    }
    
    public void newChatMessage(Message m) {
        if (m.message != null && m.message != "") {
	    if (theFrame == null || theFrame.isVisible() == false) unseenChat = true;
	    chatMessages.add(m);
	    addToListModel(m);
	    updateRosterButtons();
	}
    }
    
    public void whiteboardChatMessage(String theMessage) {
        java.util.Date d = new java.util.Date();
	long time = d.getTime();
	Message m = new Message("system", "", theMessage, "", time); 
	newChatMessage(m);   
    }
    
    public void newWhiteboardMessage(WhiteboardXMessage msg, String sender) {
        System.out.println(msg.printStr());
	WhiteboardMessage tMsg = new WhiteboardMessage(sender, msg.getType(), msg.getSVG(), msg.getSVGRM(), msg.getDesc(), msg.getMessage(), msg.getToWhom());
	whiteboardChatMessage(msg.getMessage());
	
	if (whiteboardWindow != null) {
	    tMsg.isSeen = true;
	    whiteboardWindow.incomingWhiteboardMessage(sender, msg.getType(), msg.getSVG(), msg.getSVGRM(), msg.getDesc(), msg.getToWhom());
        }//msg.printAll();
	whiteboardMessages.add(tMsg);    
    }

    public void newNormalMessage(Message m) {
        if (theFrame == null || theFrame.isVisible() == false) {
	    if (m.sJID.equalsIgnoreCase(jid)) unseenNormal = true;
	}
	normalMessages.add(m);
	updateRosterButtons();
    }

    public String shortName(String lJID) {
	int index = lJID.indexOf("@");
	if (index < 0) return lJID;
	return lJID.substring(0,index);
    }
    
    public Color jidColour(String jid) {
        Color theColour = new Color(119, 85, 85);
	if (!isSubscribed()) theColour = new Color(119, 119, 119);
        if (aComm.presence.equalsIgnoreCase("offline")) theColour = new Color(119, 119, 119);
	if (pType == null || pType.length() < 1 || pType.equalsIgnoreCase("available") || pType.equalsIgnoreCase("online")) { 
	    if (pShowState != null && pShowState.length() > 0) 	{
		theColour = new Color(68, 102, 68);
	    } else {
	        theColour = new Color(51, 153, 0);    
	    }
	}	
	return theColour;
    }
    
    public String availability(String jid) {
    	String line = "offline";
	if (!isSubscribed()) line = "*";
	if (aComm.presence.equalsIgnoreCase("offline")) line = "offline";	
	if (pType == null || pType.length() < 1 || pType.equalsIgnoreCase("available") || pType.equalsIgnoreCase("online")) { 
	    if (pShowState != null && pShowState.length() > 0) 	line = pShowState;    
	    else line = "online";	    
	}
	return line;
    }

    public ImageIcon getChatIcon() {
        if (theFrame == null || !theFrame.isVisible()) if (unseenChat) return aComm.toolKit.chatSIcon;
	return aComm.toolKit.chatIcon;
    }

    public ImageIcon getNormalIcon() {
        if (unseenNormal) return aComm.toolKit.normalSIcon;
	return aComm.toolKit.normalIcon;
    }

    public String getChatText() {
        if (theFrame == null || !theFrame.isVisible()) if (unseenChat) return "Incoming chat event from "+shortName(jid);
	return "Private chat with "+shortName(jid);
    }
    
    public String getNormalText() {
        if (unseenNormal) return "Incoming message from "+shortName(jid);
	return "Compose message to "+shortName(jid);
    }
                
    public void updateRosterButtons() {
	jidLabel.setText(shortName(jid) + " (" + availability(shortName(jid)) + ")");
	jidLabel.setForeground(jidColour(shortName(jid)));
        chatButton.setIcon(getChatIcon());
	chatButton.setToolTipText(getChatText());
	chatButton.updateUI();
	normalButton.setIcon(getNormalIcon());	
	normalButton.setToolTipText(getNormalText());
	normalButton.updateUI();
    }   

    // *********************************************************    
    // *********************************************************
    // *********************************************************
    // handle subscriptions
    public void makeSubscribed() {
        subscription = "subscribed";
	aComm.rosterHash.rosterP.setProperty(jid, "subscribed");
	aComm.rosterHash.saveProperties();
    }
    
    public void makeUnsubscribed() {
        subscription = "unsubscribed";
	aComm.rosterHash.rosterP.setProperty(jid, "unsubscribed");
	aComm.rosterHash.saveProperties();
    }
    
    public boolean isSubscribed() {
        if (subscription.equalsIgnoreCase("subscribed")) return true;
	if (subscription.equalsIgnoreCase("unsubscribed")) return false;
	return false;
    }
    
    public void getSubscription() {
        //System.out.println("JID: "+jid);
        subscription = aComm.rosterHash.rosterP.getProperty(jid);
	//System.out.println("Subsciption: "+subscription);
	if (subscription == null) subscription = "unknown";
    } 

}






