package ca.utoronto.atrc.whiteboard;

// Copyright 2003, Adaptive Technology Resource Centre
// Written by: Taras Kowaliw, taras.kowaliw@utoronto.ca

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class RosterGUI {

    public JFrame theFrame;
    public JPanel rosterPanel;
    public JPanel presencePanel;
    public JPanel groupChatPanel;
    public JScrollPane theScrollPane;
    public GridBagLayout gridbag;
    public GridBagConstraints c;
    public JLabel aLabel;
    
    // sizes
    public int rosterWidth = 250;
    public int rosterHeight = 450;
   
    public JButton joinChatButton;
    
    // presence items
    public JLabel presenceLabel;
    public JComboBox presenceBox;
    public JRadioButton p1R;    
    public JRadioButton p2R;    
    public JRadioButton p3R;  
    public JRadioButton p4R;
    public JRadioButton p5R;    
        
    // menu
    public JMenuBar menuBar;
    public JMenu aCommMenu;
    public JMenu rosterMenu;   
    
    // A-Comm menu items
    public JMenuItem aboutItem;
    public JMenuItem myJIDItem;
    public JMenuItem optionsItem;
    public JMenuItem helpItem;
    public JMenuItem exitItem;
    
    // Roster menu items
    public JMenuItem contactItem;
    public JMenuItem subscribeItem;
    public JMenuItem removeItem;
             
    // utility variables
    public int yLoc = 0;
    
    public ACommunicator aComm;
    public Properties rosterP;
    
    public RosterGUI(ACommunicator aC) {
       aComm = aC;
       theFrame = new JFrame("AComm: Roster");
       theFrame.addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent e) {
	       aComm.killItAll();
           }});
       
       makeMenus();
	   
       rosterPanel = new JPanel();         
       rosterPanel.setBackground(Color.white); 
       rosterPanel.setMinimumSize(new Dimension(rosterWidth - 50, 1500));
       drawRosterPanel();
            
       makePresencePanel();
       makeGroupChatPanel(); 
                            
       theScrollPane = new JScrollPane(rosterPanel);
       theScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
       theScrollPane.setMinimumSize(new Dimension(rosterWidth - 50, rosterHeight - 50));
       theScrollPane.setBorder(BorderFactory.createTitledBorder("Roster List")); 
       groupChatPanel.setBorder(BorderFactory.createTitledBorder("Group Chat"));       	 
       presencePanel.setBorder(BorderFactory.createTitledBorder("My Presence"));

       rosterPanelLayout();       
       theFrame.setSize(rosterWidth,rosterHeight);
       theFrame.setVisible(true); 
       
    }
    
    public void rosterPanelLayout() {
       gridbag = new GridBagLayout();
       c = new GridBagConstraints();
       theFrame.getContentPane().setLayout(gridbag);
       c.fill = GridBagConstraints.HORIZONTAL;
       c.anchor = GridBagConstraints.NORTHEAST;
       	
       c.anchor = GridBagConstraints.NORTH;
       c.weightx = 0.2;
       c.weighty = 1.0;
       c.gridwidth = 1;
       c.gridheight = 3;
       c.gridx = 0;
       c.gridy = 0;
       c.ipadx = 0;
       gridbag.setConstraints(theScrollPane, c);
       theFrame.getContentPane().add(theScrollPane);  
       
       c.anchor = GridBagConstraints.NORTH;
       c.weightx = 0.2;
       c.weighty = 0.0;
       c.gridwidth = 1;
       c.gridheight = 1;
       c.gridx = 0;
       c.gridy = 3;
       c.ipadx = 0;
       gridbag.setConstraints(presencePanel, c);
       theFrame.getContentPane().add(presencePanel); 
       
       c.anchor = GridBagConstraints.NORTH;
       c.weightx = 0.2;
       c.weighty = 0.0;
       c.gridwidth = 1;
       c.gridheight = 1;
       c.gridx = 0;
       c.gridy = 4;
       c.ipadx = 0;
       gridbag.setConstraints(groupChatPanel, c);
       theFrame.getContentPane().add(groupChatPanel);    
    }

    // Add Contact GUI Items
    private JFrame addContactFrame;
    private JTextField addJIDField;
    private JCheckBox subscribeBox;
    
        
    public void addContact() {
        addContactFrame = new JFrame("AComm: Add Contact");
	addContactFrame.getContentPane().setLayout(new BorderLayout());

	JLabel addJIDLabel = new JLabel("Enter the JID of the contact:");	       
	addJIDField = new JTextField(15);
	JPanel topPanel = new JPanel();
	topPanel.setLayout(new FlowLayout());
	topPanel.add(addJIDLabel);
	topPanel.add(addJIDField);
	
	JLabel subscribeLabel = new JLabel("Request subscription to this contact's presence information (when they are online / offline).");
	subscribeBox = new JCheckBox();
	subscribeBox.setSelected(false);
	JPanel middlePanel = new JPanel();
	middlePanel.setLayout(new FlowLayout());
	middlePanel.add(subscribeBox);
	middlePanel.add(subscribeLabel);
			       
	JButton addButton = new JButton("Add Contact");
	addButton.setMnemonic(KeyEvent.VK_A);
        addButton.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           String addJID = addJIDField.getText();
	           if (aComm.presence.equalsIgnoreCase("offline") || aComm.presence.equalsIgnoreCase("unavailable")) {
		       JOptionPane.showMessageDialog(rosterPanel, "You must be connected to add a contact. In the Roster Window, select 'Online' (Alt + O) to connect.");
		   } else if (addJID != null && addJID.length() > 0) {
	               aComm.addContact(addJID);
		       if (subscribeBox.isSelected()) {
		           aComm.subscribeContact(addJID);
		       }
		       addContactFrame.setVisible(false);
		       addContactFrame.dispose();
		   } else {
		       JOptionPane.showMessageDialog(rosterPanel, "Error: You must enter a JID to proceed.");
		   }
	       }
           });     
	
	JButton cancelButton = new JButton("Cancel");
	cancelButton.setMnemonic(KeyEvent.VK_C);
        cancelButton.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           addContactFrame.setVisible(false);
		   addContactFrame.dispose();
	       }
           });     
		       
	JPanel bottomPanel = new JPanel();
	bottomPanel.setLayout(new FlowLayout());
	bottomPanel.add(addButton);
	bottomPanel.add(cancelButton);
		       
	addContactFrame.getContentPane().add(topPanel, "North");	
	addContactFrame.getContentPane().add(middlePanel, "Center");	       
	addContactFrame.getContentPane().add(bottomPanel, "South");	              
	addContactFrame.pack();
	addContactFrame.setVisible(true);
    }
    
    public void makeMenus() {
       menuBar = new JMenuBar();
       theFrame.setJMenuBar(menuBar);
       
       aCommMenu = new JMenu("A-Comm");
       aCommMenu.setMnemonic(KeyEvent.VK_A);
       menuBar.add(aCommMenu);

       rosterMenu = new JMenu("Roster");
       rosterMenu.setMnemonic(KeyEvent.VK_R);
       menuBar.add(rosterMenu);       
       
       contactItem = new JMenuItem("Add Contact",KeyEvent.VK_A);
       contactItem.getAccessibleContext().setAccessibleDescription("Add a new Contact to the Roster");
       rosterMenu.add(contactItem);
       contactItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           if (aComm.presence.equalsIgnoreCase("offline") || aComm.presence.equalsIgnoreCase("unavailable")) {
		       JOptionPane.showMessageDialog(rosterPanel, "You must be connected to add a contact. Select 'Online' (Alt + O) to connect.");
		   } else {
	               //String addJID = JOptionPane.showInputDialog(rosterPanel, "<html><body><p>Enter the JID of the contact you wish to add:</p></body></html>");
		       //if (addJID != null && addJID.length() > 0) aComm.addContact(addJID);
		       addContact();
		   }
	       }
           });       

       subscribeItem = new JMenuItem("Subscribe",KeyEvent.VK_S);
       subscribeItem.getAccessibleContext().setAccessibleDescription("Subscribe to a Contact's Presence");
       rosterMenu.add(subscribeItem);
       subscribeItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           if (aComm.presence.equalsIgnoreCase("offline") || aComm.presence.equalsIgnoreCase("unavailable")) {
		       JOptionPane.showMessageDialog(rosterPanel, "You must be connected to subscribe to a contact's presence. Select 'Online' (Alt + O) to connect.");
		   } else {
		       String tJID = "";
		       int size = aComm.rosterHash.theHash.size();
		       int i = 0;
		       Object[] possibilities = new Object[size];
		       Enumeration enum = aComm.rosterHash.theHash.keys();
                       while (enum.hasMoreElements()) {
                           tJID = (String) enum.nextElement();
			   possibilities[i] = tJID;
			   i++;
                       }
		       if (possibilities[0] != null) {
                           tJID = (String) JOptionPane.showInputDialog(rosterPanel, "Select a Contact below to subscribe to his/her Presence. If authorized, this means that you will be notified when the Contact is online or offline, and vica versa.", "Subscribe to a Contact's Presence", JOptionPane.PLAIN_MESSAGE, null, possibilities, possibilities[0]);
		       } else {
		           JOptionPane.showMessageDialog(rosterPanel, "There are no contacts in your Roster. You will need to 'Add a Contact' before subscribing to their Presence.");
		       }
		       if (tJID != null && tJID.length() > 0) {
		           aComm.subscribeContact(tJID);
		       }
		   }
	       }
           }); 
       
       removeItem = new JMenuItem("Remove a Contact",KeyEvent.VK_R);
       removeItem.getAccessibleContext().setAccessibleDescription("Remove a Contact from your Roster");
       rosterMenu.add(removeItem);
       removeItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           if (aComm.presence.equalsIgnoreCase("offline") || aComm.presence.equalsIgnoreCase("unavailable")) {
		       JOptionPane.showMessageDialog(rosterPanel, "You must be connected to remove a Contact. Select 'Online' (Alt + O) to connect.");
		   } else {
		       String tJID = "";
		       int size = aComm.rosterHash.theHash.size();
		       int i = 0;
		       Object[] possibilities = new Object[size];
		       Enumeration enum = aComm.rosterHash.theHash.keys();
                       while (enum.hasMoreElements()) {
                           tJID = (String) enum.nextElement();
			   possibilities[i] = tJID;
			   i++;
                       }
		       if (possibilities[0] != null) {
                           tJID = (String) JOptionPane.showInputDialog(rosterPanel, "Select a Contact below to remove him/her from your Roster.", "Subscribe to a Contact's Presence", JOptionPane.PLAIN_MESSAGE, null, possibilities, possibilities[0]);
		       } else {
		           JOptionPane.showMessageDialog(rosterPanel, "There are no contacts in your Roster.");
		       }
		       if (tJID != null && tJID.length() > 0) {
		           aComm.removeContact(tJID);
		       }
		   }
	       }
           }); 	   	          	
       
       optionsItem = new JMenuItem("Options",KeyEvent.VK_O);
       optionsItem.getAccessibleContext().setAccessibleDescription("A-Communicator Options");
       aCommMenu.add(optionsItem);
       optionsItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           aComm.showOptionGUI();
	           //JOptionPane.showMessageDialog(rosterPanel, "<html><body>Options screen goes here</body></html>");
	       }
           });
          
       aboutItem = new JMenuItem("About",KeyEvent.VK_A);
       aboutItem.getAccessibleContext().setAccessibleDescription("About A-Communicator");
       aCommMenu.add(aboutItem);
       aboutItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           JOptionPane.showMessageDialog(rosterPanel, "XXX DEVELOPMENTAL ALPHA VERSION --- DO NOT DISTRIBUTE XXX. AComm was developed by the Adaptive Technology Resources Centre at the University of Toronto, Copyright 2004, ATRC. Please visit http://acomm.atrc.utoronto.ca for more information.");
	       }
           });	
	   
       myJIDItem = new JMenuItem("My JID",KeyEvent.VK_J);
       myJIDItem.getAccessibleContext().setAccessibleDescription("Show My JID");
       aCommMenu.add(myJIDItem);
       myJIDItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           JOptionPane.showMessageDialog(rosterPanel, aComm.myJID);
	       }
           });	

       helpItem = new JMenuItem("Help",KeyEvent.VK_H);
       helpItem.getAccessibleContext().setAccessibleDescription("Show Help Screen");
       aCommMenu.add(helpItem);
       helpItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           JOptionPane.showMessageDialog(rosterPanel, "Help screen goes here");
	       }
           });
   	          
       exitItem = new JMenuItem("Exit",KeyEvent.VK_X);
       exitItem.getAccessibleContext().setAccessibleDescription("Exit A-Communicator");
       aCommMenu.add(exitItem);
       exitItem.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           aComm.killItAll();
	       }
           });    
    }
    
    public void makePresencePanel() {
       presencePanel = new JPanel();
       presencePanel.setSize(rosterWidth, 150);
       presencePanel.setLayout(new BoxLayout(presencePanel, BoxLayout.Y_AXIS));
       //presencePanel.setBackground(Color.red);
       /*presenceLabel = new JLabel("<html><body>My Presence: </body></html>");
       presencePanel.add(presenceLabel);*/

       ButtonGroup group2 = new ButtonGroup();
       p1R = new JRadioButton("Online");
       p1R.setForeground(new Color(51, 153, 0));
       p1R.setMnemonic(KeyEvent.VK_O);
       p1R.setDisplayedMnemonicIndex(0);
       if (aComm.presence.equalsIgnoreCase("online")) p1R.setSelected(true);
       p1R.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           aComm.setOnline();
	       }
           });
            
       p2R = new JRadioButton("Away");
       p2R.setForeground(new Color(68, 102, 68));
       p2R.setMnemonic(KeyEvent.VK_W);
       if (aComm.presence.equalsIgnoreCase("away")) p2R.setSelected(true);
       p2R.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           aComm.setAway();
	       }
           });

       //p3R = new JRadioButton("<html><body><table border='0' cellpadding='0' cellspacing='0'><tr><td width='"+(rosterWidth-50)+"'><font color='446644'>E<u>x</u>tended Away</font></td></tr></table></body></html>");
       p3R = new JRadioButton("Extended Away");
       p3R.setForeground(new Color(68, 102, 68));
       p3R.setMnemonic(KeyEvent.VK_X);
       if (aComm.presence.equalsIgnoreCase("xa")) p3R.setSelected(true);
       p3R.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           aComm.setNA();
	       }
           });	   
       
       p4R = new JRadioButton("Do Not Disturb");
       p4R.setForeground(new Color(68, 102, 68));
       p4R.setMnemonic(KeyEvent.VK_D);
       if (aComm.presence.equalsIgnoreCase("dnd")) p4R.setSelected(true);
       p4R.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           aComm.setDND();
	       }
           });	   
       
       p5R = new JRadioButton("Offline");
       p5R.setForeground(new Color(119, 85, 85));
       p5R.setMnemonic(KeyEvent.VK_F);
       if (aComm.presence.equalsIgnoreCase("offline")) p5R.setSelected(true);
       p5R.addActionListener(new ActionListener() {
               public void actionPerformed(ActionEvent e) {
	           aComm.setOffline();
	       }
           });
       
       p1R.setBorder(BorderFactory.createEmptyBorder(0,20,0,0));
       p2R.setBorder(BorderFactory.createEmptyBorder(0,20,0,0));
       p3R.setBorder(BorderFactory.createEmptyBorder(0,20,0,0));
       p4R.setBorder(BorderFactory.createEmptyBorder(0,20,0,0));
       p5R.setBorder(BorderFactory.createEmptyBorder(0,20,0,0));
       group2.add(p1R);              
       group2.add(p2R);
       group2.add(p3R);              
       group2.add(p4R);
       group2.add(p5R);       
       presencePanel.add(p1R);
       presencePanel.add(p2R);
       presencePanel.add(p3R);
       presencePanel.add(p4R);
       presencePanel.add(p5R);    
    }
    
    
    public void makeGroupChatPanel() {
    
    	groupChatPanel = new JPanel();
	groupChatPanel.setLayout(new BorderLayout());
	joinChatButton = new JButton("Join Group Chat");
	joinChatButton.setMnemonic(KeyEvent.VK_G);
	joinChatButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
		if (aComm.presence.equalsIgnoreCase("offline") || aComm.presence.equalsIgnoreCase("unavailable")) {
		    JOptionPane.showMessageDialog(rosterPanel, "You must be connected to connect to a group chat. Select 'Online' (Alt + O) to connect.");
	        } else {
	            ACommGroupChatLogin gCLogin = new ACommGroupChatLogin(aComm);
		}
           }});
	groupChatPanel.add(joinChatButton, "Center");
	    
    }
 
    public void addRoster(Roster theRoster) {
	
        // add the label
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 0.2;
        c.weighty = 0.0;
        c.gridx = 0;
        c.gridy = yLoc;
        c.ipadx = 0;
        gridbag.setConstraints(theRoster.jidLabel, c);
        rosterPanel.add(theRoster.jidLabel);  
	
	// add the chat button
        theRoster.chatButton.setMargin(new Insets(0,0,0,0));
        theRoster.chatButton.setIconTextGap(0);
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 0.2;
        c.weighty = 0.0;
        c.gridx = 1;
        c.gridy = yLoc;
        c.ipadx = 0;
        gridbag.setConstraints(theRoster.chatButton, c);
        rosterPanel.add(theRoster.chatButton);  
	
	// add the normal button
        theRoster.normalButton.setMargin(new Insets(0,0,0,0));
        theRoster.normalButton.setIconTextGap(0);	
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 0.2;
        c.weighty = 0.0;
        c.gridx = 2;
        c.gridy = yLoc;
        c.ipadx = 0;
        gridbag.setConstraints(theRoster.normalButton, c);
        rosterPanel.add(theRoster.normalButton);  
	 
        yLoc ++;
    }
    
    public void drawRosterPanel() {
       rosterPanel.removeAll();
    
       gridbag = new GridBagLayout();
       c = new GridBagConstraints();
       rosterPanel.setLayout(gridbag);
       c.fill = GridBagConstraints.HORIZONTAL;
       c.anchor = GridBagConstraints.NORTHWEST;
              
       Enumeration enum = aComm.rosterHash.theHash.keys();
       
       while (enum.hasMoreElements()) {
           String tJID = (String) enum.nextElement();
	   Roster temp = aComm.rosterHash.getRoster(tJID);
	   addRoster(temp);

       }
       
       aLabel = new JLabel(getALabel());
       c.gridy = 100;
       c.gridx = 0;
       c.weighty = 1.0;
       c.anchor = GridBagConstraints.SOUTHEAST;
       gridbag.setConstraints(aLabel, c);
       rosterPanel.add(aLabel); 
       
       if (theFrame.isVisible()) rosterPanel.updateUI();   
    }
    
    public String getALabel() {
       int num = 20 - aComm.rosterHash.theHash.size();
       String theString = "<html><body>";
       for (int i=0; i<num; i++) theString += "<br>";
       theString += "</body></html>\n";
       return theString;
    }
    
    public String jidTable(String jid) {
        return "<html><body text='"+aComm.toolKit.getColourStr(jid)+"'><table cellpadding='5' cellspacing='0' border='0' width='100'><tr><td><b>"+jid+"</b></td></tr></table></body></html>";
    }    
    
    public String getNiceJID(String tJID) {
	String temp = tJID;
	int index = temp.indexOf("@");
	if (index > 0) temp = temp.substring(0,index);
	return temp;
    }
    
}

