package ca.utoronto.atrc.whiteboard;

// Copyright 2003, Adaptive Technology Resource Centre
// Written by: Taras Kowaliw, taras.kowaliw@utoronto.ca

// RosterHash:
// A class which manages the contact list, including a Hash for the contacts and
// and HTML-based GUI for calling chats and message windows

import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Properties;
import java.net.URL;

public class RosterHash {

    public Hashtable theHash;
    public Hashtable tempHash;
    public ACommunicator aComm;
    
    public Properties rosterP;
    public String fileName;

    public RosterHash(ACommunicator aC) {
	aComm = aC;
	fileName = System.getProperty("user.home")+ File.separator + ".drawswf" + File.separator + aComm.myJID + ".roster.properties";
	theHash = new Hashtable();
	tempHash = new Hashtable();
	rosterP = new Properties();
	loadProperties();
	
	Enumeration e = rosterP.propertyNames();
	while (e.hasMoreElements()) {
	    String tJID = (String) e.nextElement();
	    String subscription = rosterP.getProperty(tJID);
	    Roster temp = new Roster(tJID, aComm, subscription);
	    theHash.put(tJID, temp);
	}
    }
    
    public void saveProperties() {
	try {
            FileOutputStream out = new FileOutputStream(new File(fileName));
	    rosterP.store(out, "ACommunicator Roster - do not modify");	
	} catch (Exception e) { e.printStackTrace(); }    
    }
    
    public void removeProperty(String tJID) {
	try {
	    BufferedReader in = new BufferedReader(new FileReader(fileName));
	    String line = "";
	    String newFile = "";
	    while ((line = in.readLine()) != null) if (!line.startsWith(tJID+"=")) newFile += line + "\n";
	    in.close();
	    PrintWriter out = new PrintWriter(new FileWriter(fileName));
	    out.print(newFile);
	    out.close();
	} catch (Exception e) { e.printStackTrace(); }      
	loadProperties();
    }
    
    public void loadProperties() {
        rosterP = null;
	rosterP = new Properties();
	try {
	    FileInputStream in = new FileInputStream(new File(fileName));
	    rosterP.load(in);	
	} catch (Exception e) {
	    saveProperties();
	}    
    }

    public void setPresence(String tJID, String pType, String pStatus, String pShowState) {
	Roster temp = ((Roster) theHash.get(tJID));
	if (temp == null) {
	    tempHash.put(tJID, pType);
	} else {
	    boolean wasOnline = (temp.pType == null || temp.pType.equalsIgnoreCase("available") || temp.pType.equalsIgnoreCase("online"));
	    temp.pType = pType;
	    temp.pStatus = pStatus;
	    temp.pShowState = pShowState; 
	    boolean goingOnline = (temp.pType == null || temp.pType.equalsIgnoreCase("available") || temp.pType.equalsIgnoreCase("online"));
	    if (goingOnline) temp.subscription = "subscribed";
	    temp.updateRosterButtons();
	    if (!wasOnline && goingOnline && aComm.jidOptions.getBoolean("playOnlineSound")) aComm.toolKit.playOnlineSound();
	    else if (!goingOnline && wasOnline && aComm.jidOptions.getBoolean("playOfflineSound")) aComm.toolKit.playOfflineSound();
	}
    }

    public void addRosterItem(com.echomine.jabber.RosterItem rI) {
        String tempJID = rI.getJID();
	String tempSub = rI.getSubscription();
	Roster temp = (Roster) theHash.get(tempJID);
	if (temp == null) {
	    temp = new Roster(rI, aComm);
	    theHash.put(tempJID, temp);
	    rosterP.setProperty(tempJID, "");
	    saveProperties();
	    aComm.rosterGUI.drawRosterPanel();
	} else {
	    temp.rosterItem = rI;
	    if (tempSub.equalsIgnoreCase("both") || tempSub.equalsIgnoreCase("subscribed")) temp.makeSubscribed();
	    else temp.makeUnsubscribed();
	}
	String tPType = (String) tempHash.get(tempJID); 
	if (tPType != null) {
	    temp.pType = tPType;
	    tempHash.remove(tempJID);
	}
	if (temp != null) temp.updateRosterButtons();
    }

    public void addRosterItem(String addJID, String sub) {
	Roster temp = new Roster(addJID, sub, aComm);
	theHash.put(addJID, temp);
	rosterP.setProperty(addJID, sub);
	saveProperties();
	aComm.rosterGUI.drawRosterPanel();
	temp.updateRosterButtons();
    } 
    
    public void remove(String tempJID) {
        Roster temp = (Roster) theHash.get(tempJID);
	if (temp != null) {
	    theHash.remove(tempJID);
	    removeProperty(tempJID);
	    aComm.rosterGUI.drawRosterPanel();
	}
    }   
    
    public void setAllUnsubscribed() {
	Enumeration enum = theHash.keys();
	while (enum.hasMoreElements()) {
	    String tJID = (String) enum.nextElement();
	    Roster temp = (Roster) theHash.get(tJID);
	    temp.pType = "offline";
	    temp.updateRosterButtons();
	}    
    }
    
    public void printAll() {
	Enumeration enum = theHash.keys();
	while (enum.hasMoreElements()) {
	    String tJID = (String) enum.nextElement();
	    Roster temp = (Roster) theHash.get(tJID);
	    System.out.println("Key: "+tJID+", "+temp.pType+", "+temp.subscription);
	}
    }
    
    public Roster getRoster(String jid) {
        Roster temp = (Roster) theHash.get(jid);
	return temp;
    }

    public String getNiceJID(String tJID) {
	String temp = tJID;
	int index = temp.indexOf("@");
	if (index > 0) temp = temp.substring(0,index);
	return temp;
    }

    public String fontRegular(String text) {
	return text;
    }
    public String fontSmall(String text) {
	return text;
    }

}
