package ca.utoronto.atrc.whiteboard;

import com.echomine.jabber.*;
import com.echomine.common.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;
import org.jdom.*;
import java.util.List;
import java.util.Iterator;
import java.util.Vector;
import java.io.*;
import org.jdom.input.*;
import org.jdom.output.*;

public class WhiteboardXMessage extends JabberJDOMMessage {
    private String text;
    private String from;
    private String time;
    private SimpleDateFormat format;

    public String whiteboardType;
    public Element svgTree;
    public Element svgRMTree;
    public Element descListTree;   
    
    public Vector svgStrings;
    public Vector svgRMStrings;
    public Vector descObjects;
    
    public String messageStr;
    
    public String type; 
    public String toWhom;
    
    /** constructs a default Whiteboard message */
    public WhiteboardXMessage() {
        super(new Element("x", Namespace.getNamespace("atrc:utoronto:ca:whiteboard")));
        //super(new Element("x", Namespace.getNamespace("atrc:utoronto:ca:whiteboard")));
    }

    /** constructs a default Whiteboard message (with the x element) */
    public WhiteboardXMessage(Element x) {
        super(x);
    }    


    // the String is an indent used for debugging... since I'm using this 
    // heavily, I won't remove it. Just include an empty string (i.e. "")
    public void runThroughTree(Element top, String indent) {
        List allChildren = top.getChildren();
	Iterator it = allChildren.iterator();
	while (it.hasNext()) {
	    Element temp = (Element) it.next();
	    String name = temp.getName();
	    //System.out.println(indent+name);
	    if (name.equalsIgnoreCase("whiteboard")) {
	        Attribute wAtt = temp.getAttribute("type");
		if (wAtt != null) whiteboardType = wAtt.getValue();
	        Attribute toAtt = temp.getAttribute("who");
		if (toAtt != null) toWhom = toAtt.getValue();
	    } else if (name.equalsIgnoreCase("svg")) {
	        svgTree = temp;
	    } else if (name.equalsIgnoreCase("svg_rm")) {
	        svgRMTree = temp;
	    } else if (name.equalsIgnoreCase("descList")) {
	        descListTree = temp;
	    } else if (name.equalsIgnoreCase("chat_message")) {
	        messageStr = temp.getText();
	    }
	    runThroughTree(temp, indent+"   ");
	}		
    }
    
    public Vector getSVG() {
        return svgStrings;
    }
    
    public Vector getSVGRM() {
        return svgRMStrings;
    }
    
    public Vector getDesc() {
        return descObjects;
    }
    
    public String getType() {
        return whiteboardType;
    }

    public String getToWhom() {
        return toWhom;
    }    
        
    public String getMessage() {
        return messageStr;
    }

    public JabberMessage parse(JabberMessageParser parser, Element msgTree) throws ParseException {
	
	whiteboardType = null;
	svgTree = null;
	svgRMTree = null;
	descListTree = null;

	runThroughTree(msgTree,"");
	
	// produce svg Vector
	Iterator it;
	svgStrings = null;
	if (svgTree != null) {
	    svgStrings = new Vector();
	    List svgKids = svgTree.getChildren();
	    it = svgKids.iterator();
	    while(it.hasNext()) {
	        Element temp = (Element) it.next();
	        svgStrings.add(elementToString(temp));
	    }
	} 
	svgRMStrings = null;
	if (svgRMTree != null) {
	    svgRMStrings = new Vector();
	    List svgRMKids = svgRMTree.getChildren();
	    it = svgRMKids.iterator();
	    while(it.hasNext()) {
	        Element temp = (Element) it.next();
	        svgRMStrings.add(elementToString(temp));
	    }
	} 
	descObjects = null;
	if (descListTree != null) {
	    descObjects = new Vector();
	    List descListKids = descListTree.getChildren();
	    it = descListKids.iterator();
	    while(it.hasNext()) {
	        Element temp = (Element) it.next();
	        descObjects.add(elementToString(temp));
	    }
	} 
	
        super.parse(parser, msgTree);
	this.from = msgTree.getAttributeValue("from");
        time = msgTree.getAttributeValue("stamp");
        this.text = msgTree.getText();
	return this;
    }
    
    public void printAll() {
        /*System.out.println("Whiteboard Internals:");
	System.out.println("Type: "+whiteboardType);
	if (svgTree == null) System.out.println("svgTree is null");
	else System.out.println("svgTree: "+svgTree.getName());
	if (svgRMTree == null) System.out.println("svgRMTree is null");
	else System.out.println("svgTree: "+svgRMTree.getName());
	if (descListTree == null) System.out.println("descListTree is null");
	else System.out.println("descListTree: "+descListTree.getName());*/	
    }    

    public String printStr() {
        String retStr = "";
	retStr += "Whiteboard Internals:\n";
	retStr += "   Type: "+whiteboardType+"\n";
	if (svgTree == null) retStr += "   svgTree is null"+"\n";
	else retStr += "   svgTree: "+svgTree.getName()+"\n";
	if (svgRMTree == null) retStr += "   svgRMTree is null"+"\n";
	else retStr += "   svgTree: "+svgRMTree.getName()+"\n";
	if (descListTree == null) retStr += "  descListTree is null"+"\n";
	else retStr += "   descListTree: "+descListTree.getName()+"\n";	
	if (messageStr == null)  retStr += "  messageStr is null"+"\n";
	else retStr += "   messageStr: "+messageStr+"\n";	
	return retStr;
    }    
    
    public String elementToString(Element e) {
        String theString = null;
	try {
	    XMLOutputter xmlOut = new XMLOutputter();
	    theString = xmlOut.outputString(e);
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
	return theString;
    }  
    
    public Element stringToElement(String s) {
        Element e = null;
    	try {
	    StringReader sR = new StringReader(s);
            SAXBuilder sB = new SAXBuilder();
	    Document doc = sB.build(sR);
	    e = doc.getRootElement();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
	return e;
    }  
    
    /**
     * The time when the message was sent by the sender, NOT the time received.
     * Note that the time is local to your time zone, not in UTC or the sender's
     * timezone.  Thus, if you are in PST (GMT-800), then you will get when the message was sent in PST time.
     * @return the time the message was sent, null if none exists
     */
    public Date getTimeInLocal() {
        if (time == null) return null;
        if (format == null) {
            format = new SimpleDateFormat("yyyyMMdd'T'HH:mm:ss");
            format.setTimeZone(TimeZone.getTimeZone("GMT"));
        }
        Date date;
        date = format.parse(time, new ParsePosition(0));
        return date;
    }

    /**
     * The time stamp string as returned by the remote server. This time stamp is in UTC.
     * The time stamp is formatted as an ISO date format.  To parse it, you can use something like:
     * SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd'T'HH:mm:ss");
     * @return the time stamp string
     */
    public String getTime() {
        return time;
    }

    /** @return the sender of the message, null if none exists */
    public String getFrom() {
        return from;
    }

    /**
     * the text that is optionally associated with the message.
     * @return the text message or null if none exists
     */
    public String getText() {
        return text;
    }

    public int getMessageType() {
        return 6543;
        //return JabberCode.MSG_X_DELAY;
    }
}
